import re

from numpy import asarray

from openfisca_france.model.base import *


class date_naissance(Variable):
    value_type = date
    default_value = date(1970, 1, 1)
    entity = Individu
    label = 'Date de naissance'
    definition_period = ETERNITY


class majeur(Variable):
    value_type = bool
    entity = Individu
    label = "L'individu est considéré comme majeur"
    definition_period = MONTH

    def formula(individu, period, parameters):
        majeur = individu('age', period) >= parameters(period).geopolitique.age_majorite
        mineur_emancipe = individu('mineur_emancipe', period)

        return majeur + mineur_emancipe


class mineur_emancipe(Variable):
    value_type = bool
    default_value = False
    entity = Individu
    label = "L'individu est émancipé"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class orphelin(Variable):
    value_type = bool
    default_value = False
    entity = Individu
    label = "L'individu est orphelin de ses deux parents"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class adoption(Variable):
    value_type = bool
    entity = Individu
    label = 'Enfant adopté'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class garde_alternee(Variable):
    value_type = bool
    entity = Individu
    label = 'Enfant en garde alternée'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class plus_haut_diplome_niveau(Variable):
    value_type = Enum
    default_value = TypesNiveauDiplome.non_renseigne
    possible_values = TypesNiveauDiplome  # defined in model/base.py
    entity = Individu
    label = 'Plus haut niveau de diplôme obtenu'
    definition_period = MONTH


class plus_haut_diplome_date_obtention(Variable):
    value_type = date
    default_value = date.max
    entity = Individu
    label = "Date d'obtention du diplôme de plus haut niveau"
    definition_period = MONTH


class alternant(Variable):
    value_type = bool
    label = 'En formation en alternance'
    entity = Individu
    definition_period = MONTH
    set_input = set_input_dispatch_by_period
    reference = 'https://www.service-public.fr/particuliers/vosdroits/N11240'


class niveau_diplome_formation(Variable):
    value_type = Enum
    default_value = TypesNiveauDiplome.non_renseigne
    possible_values = TypesNiveauDiplome  # defined in model/base.py
    entity = Individu
    label = 'Niveau du diplôme en cours de préparation'
    definition_period = MONTH


class activite(Variable):
    value_type = Enum
    default_value = TypesActivite.inactif
    possible_values = TypesActivite  # defined in model/base.py
    entity = Individu
    label = 'Activité'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class enceinte(Variable):
    value_type = bool
    entity = Individu
    label = 'Est enceinte'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class statut_marital(Variable):
    value_type = Enum
    possible_values = TypesStatutMarital  # defined in model/base.py
    default_value = TypesStatutMarital.celibataire
    entity = Individu
    label = 'Statut marital'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period, parameters):
        # Par défault, on considère que deux adultes dans un foyer fiscal sont PACSÉS
        deux_adultes = individu.foyer_fiscal.nb_persons(FoyerFiscal.DECLARANT) >= 2
        return where(deux_adultes, TypesStatutMarital.pacse, TypesStatutMarital.celibataire)


class nbN(Variable):
    cerfa_field = 'N'
    value_type = int
    is_period_size_independent = True
    entity = FoyerFiscal
    label = "Nombre d'enfants mariés/pacsés et d'enfants non mariés chargés de famille"
    definition_period = YEAR


class nbR(Variable):
    cerfa_field = 'R'
    value_type = int
    is_period_size_independent = True
    entity = FoyerFiscal
    label = "Nombre de titulaires (autres que les enfants) de la carte invalidité d'au moins 80 %"
    definition_period = YEAR


class caseE(Variable):
    cerfa_field = 'E'
    value_type = bool
    entity = FoyerFiscal
    label = "Situation pouvant donner droit à une demi-part supplémentaire : vous vivez seul au 1er janvier de l'année de perception des revenus et vous avez élevé un enfant pendant moins de 5 ans durant la période où vous viviez seul"
    end = '2012-12-31'
    definition_period = YEAR


class caseF(Variable):
    cerfa_field = 'F'
    value_type = bool
    entity = FoyerFiscal
    label = "Situation pouvant donner droit à une demi-part supplémentaire : conjoint titulaire d'une pension ou d'une carte d'invalidité (vivant ou décédé l'année de perception des revenus)"
    definition_period = YEAR


class caseG(Variable):
    cerfa_field = 'G'
    value_type = bool
    entity = FoyerFiscal
    label = "Titulaire d'une pension de veuve de guerre"
    definition_period = YEAR
    # attention, ne pas confondre caseG et nbG qui se rapportent toutes les 2 à une "case" G, l'une étant une vraie case
    # que l'on remplt et l'autre une case que l'on coche


class annee_naissance_pac_alterne(Variable):
    value_type = int
    is_period_size_independent = True
    entity = FoyerFiscal
    label = 'Année de naissance des enfants à charge en garde alternée'
    definition_period = YEAR
    # Il s'agit de l'année de naissance associé aux PAC déclarés dans nbH. Il peut y avoir plusieurs années de naissance si nbH>1. On ne le prend pas en compte


class caseK(Variable):
    cerfa_field = 'K'
    value_type = bool
    entity = FoyerFiscal
    label = 'Situation pouvant donner droit à une demi-part supplémentaire: vous avez eu un enfant décédé après l’âge de 16 ans ou par suite de faits de guerre'
    end = '2011-12-31'
    definition_period = YEAR


class caseL(Variable):
    cerfa_field = 'L'
    value_type = bool
    entity = FoyerFiscal
    label = "Situation pouvant donner droit à une demi-part supplémentaire: vous vivez seul au 1er janvier de l'année de perception des revenus et vous avez élevé un enfant pendant au moins 5 ans durant la période où vous viviez seul (définition depuis 2009) - Un au moins de vos enfants à charge ou rattaché est issu du mariage avec votre conjoint décédé (définition avant 2008)"
    definition_period = YEAR


class caseN(Variable):
    cerfa_field = 'N'
    value_type = bool
    entity = FoyerFiscal
    label = "Vous ne viviez pas seul au 1er janvier de l'année de perception des revenus"
    definition_period = YEAR


class caseP(Variable):
    cerfa_field = 'P'
    value_type = bool
    entity = FoyerFiscal
    label = "Titulaire d'une pension pour une invalidité d'au moins 40 % ou d'une carte d'invalidité d'au moins 80%"
    definition_period = YEAR


class caseS(Variable):
    cerfa_field = 'S'
    value_type = bool
    entity = FoyerFiscal
    label = "Vous êtes mariés/pacsés et l'un des deux déclarants âgé de plus de 75 ans est titulaire de la carte du combattant ou d'une pension militaire d'invalidité ou de victime de guerre"
    definition_period = YEAR


class caseT(Variable):
    cerfa_field = 'T'
    value_type = bool
    entity = FoyerFiscal
    label = "Vous êtes parent isolé au 1er janvier de l'année de perception des revenus"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period
    # TODO: Set definition_period as YEAR and change the suggestion process (scenarios.py)


class caseW(Variable):
    cerfa_field = 'W'
    value_type = bool
    entity = FoyerFiscal
    label = "Vous ou votre conjoint (même s'il est décédé), âgés de plus de 75 ans, êtes titulaire de la carte du combattant ou d'une pension militaire d'invalidité ou de victime de guerre"
    definition_period = YEAR


class handicap(Variable):
    value_type = bool
    entity = Individu
    label = 'Individu en situation de handicap'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class invalidite(Variable):
    value_type = bool
    entity = Individu
    label = "Individu titulaire d'une carte d'invalidité"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class nb_parents(Variable):
    value_type = int
    is_period_size_independent = True
    entity = Famille
    label = "Nombre d'adultes (parents) dans la famille"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(famille, period):
        # Note : Cette variable est « instantanée » : quelle que soit la période demandée, elle retourne la valeur au premier
        # jour, sans changer la période.

        return famille.nb_persons(role = Famille.PARENT)


class maries(Variable):
    value_type = bool
    entity = Famille
    label = 'maries'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(famille, period):
        # Note : Cette variable est « instantanée » : quelle que soit la période demandée, elle retourne la valeur au premier
        # jour, sans changer la période.
        statut_marital = famille.members('statut_marital', period)
        individu_marie = (statut_marital == TypesStatutMarital.marie)

        return famille.any(individu_marie, role = Famille.PARENT)


class en_couple(Variable):
    value_type = bool
    entity = Famille
    label = 'Indicatrice de vie en couple'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(famille, period, parameters):
        # Note : Cette variable est « instantanée » : quelle que soit la période demandée, elle retourne la valeur au premier
        # jour, sans changer la période.
        nb_parents = famille('nb_parents', period)

        return nb_parents == 2


class est_enfant_dans_famille(Variable):
    value_type = bool
    entity = Individu
    label = "Indique que l'individu est un enfant dans une famille"
    definition_period = ETERNITY

    def formula(individu, period):
        return individu.has_role(Famille.ENFANT)


class etudiant(Variable):
    '''
    L'individu est inscrit·e dans un établissement en vue de la préparation d'un concours ou d'un diplôme de l'enseignement supérieur français : une université, une école de commerce ou d'ingénieur, dans un lycée pour un BTS…
    '''
    value_type = bool
    entity = Individu
    label = "Indique que l'individu dispose du statut étudiant"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period
    reference = 'https://www.service-public.fr/particuliers/vosdroits/F986'

    def formula(individu, period, parameters):
        # Note : Cette variable est « instantanée » : quelle que soit la période demandée, elle retourne la valeur au premier
        # jour, sans changer la période.
        activite = individu('activite', period)

        return activite == TypesActivite.etudiant


class rempli_obligation_scolaire(Variable):
    value_type = bool
    default_value = True
    entity = Individu
    label = "Remplit l'obligation scolaire"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class nationalite(Variable):
    value_type = str
    entity = Individu
    default_value = 'FR'
    max_length = 2
    label = "Code ISO de la nationalité de l'individu"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class ressortissant_eee(Variable):
    value_type = bool
    default_value = True
    entity = Individu
    label = "Individu ressortissant d'un pays membre de l'Espace Économique Européen (EEE)."
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period, parameters):
        nationalite = individu('nationalite', period)
        return sum([nationalite == str.encode(etat_membre) for etat_membre in parameters(period).geopolitique.eee])  # TOOPTIMIZE: string encoding into bytes array should be done at load time


class resident_ue(Variable):
    value_type = bool
    default_value = True
    entity = Individu
    label = "Individu résidant dans pays membre de l'Union européenne (UE)."
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period, parameters):
        '''
        La résidence est supposée par la nationalité.
        Si la résidence est déterminée d'une autre manière plus précise, écraser cette variable en la définissant plutôt qu'en la laissant calculer par la nationalité.
        '''
        nationalite = individu('nationalite', period)
        return sum([nationalite == str.encode(etat_membre) for etat_membre in parameters(period).geopolitique.ue])  # TOOPTIMIZE: string encoding into bytes array should be done at load time


class residence_continue_annees(Variable):
    value_type = float
    entity = Individu
    label = "Durée depuis laquelle l'individu réside sur le territoire français de manière régulière et continue (en années)"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class duree_possession_titre_sejour(Variable):
    value_type = float
    entity = Individu
    label = "Durée depuis laquelle l'individu possède un titre de séjour (en années)"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class enfant_place(Variable):
    value_type = bool
    entity = Individu
    label = "Enfant placé en structure spécialisée ou famille d'accueil"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class RegimeSecuriteSociale(Enum):
    regime_general = 'Régime général'
    regime_agricole = 'Régime agricole'
    regime_retraite_fonctionnaires_civils_militaires_etat = 'Régime de retraite des fonctionnaires civils et militaires de l’État'
    regime_special_fonctionnaires_territoriaux_hospitaliers = 'Régime spécial des fonctionnaires territoriaux et hospitaliers'
    fond_special_pensions_ouvriers_etablissements_industriels_etat = 'Le Fonds spécial des pensions des ouvriers des établissements industriels de l’État (FSPOEIE)'
    regime_special_agents_sncf = 'Régime spécial des agents de la SNCF'
    regime_special_agents_ratp = 'Régime spécial des agents de la RATP'
    regime_special_industries_electriques_gazieres = 'Régime spécial des industries électriques et gazières (CNIEG)'
    autres_regimes = 'Autres régimes'
    aucun = 'Aucun'


class regime_securite_sociale(Variable):
    value_type = Enum
    possible_values = RegimeSecuriteSociale
    default_value = RegimeSecuriteSociale.aucun
    entity = Individu
    label = 'Régime de sécurité sociale'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period
    reference = 'https://www.securite-sociale.fr/files/live/sites/SSFR/files/medias/CCSS/2020/RAPPORT%20CCSS-Sept%202020.pdf'


class GroupeSpecialitesFormation(Enum):
    groupe_100 = '100. Formations générales.',
    groupe_110 = '110. Spécialités pluriscientifiques.',
    groupe_111 = '111. Physique-chimie.',
    groupe_112 = '112. Chimie-biologie, biochimie.',
    groupe_113 = '113. Sciences naturelles (biologie-géologie).',
    groupe_114 = '114. Mathématiques.',
    groupe_115 = '115. Physique.',
    groupe_116 = '116. Chimie.',
    groupe_117 = '117. Sciences de la Terre.',
    groupe_118 = '118. Sciences de la vie.',
    groupe_120 = '120. Spécialités pluridisciplinaires. Sciences humaines et droit.',
    groupe_121 = '121. Géographie.',
    groupe_122 = '122. Economie.',
    groupe_123 = '123. Sciences (y compris démographie, anthropologie).',
    groupe_124 = '124. Psychologie.',
    groupe_125 = '125. Linguistique.',
    groupe_126 = '126. Histoire.',
    groupe_127 = '127. Philosophie, éthique et théologie.',
    groupe_128 = '128. Droit, sciences politiques.',
    groupe_130 = '130. Spécialités littéraires et artistiques plurivalentes.',
    groupe_131 = '131. Français, littérature et civilisation française.',
    groupe_132 = '132. Arts plastiques.',
    groupe_133 = '133. Musique, arts du spectacle.',
    groupe_134 = '134. Autres disciplines artistiques et spécialités artistiques plurivalentes.',
    groupe_135 = '135. Langues et civilisations anciennes.',
    groupe_136 = '136. Langues vivantes, civilisations étrangères et régionales.',
    groupe_200 = '200. Technologies industrielles fondamentales (génie industriel',
    groupe_201 = '201. Technologies de commandes des transformations industrielles',
    groupe_210 = "210. Spécialités plurivalentes de l'agronomie et de l'agriculture.",
    groupe_211 = '211. Productions végétales, cultures spécialisées et protection des cultures (horticulture, viticulture, arboriculture fruitière...).',
    groupe_212 = '212. Productions animales, élevage spécialisé, aquaculture, soins aux animaux (y compris vétérinaire).',
    groupe_213 = '213. Forêts, espaces naturels, faune sauvage, pêche.',
    groupe_214 = '214. Aménagement paysager (parcs, jardins, espaces verts, terrains de sport).',
    groupe_220 = '220. Spécialités pluritechnologiques des transformations.',
    groupe_221 = '221. Agro-alimentaire, alimentation, cuisine.',
    groupe_222 = '222. Transformations chimiques et apparentées (y compris industrie pharmaceutique).',
    groupe_223 = '223. Métallurgie (y compris sidérurgie, fonderie, non-ferreux).',
    groupe_224 = '224. Matériaux de construction, verre, céramique.',
    groupe_225 = '225. Plasturgie, matériaux composites.',
    groupe_226 = '226. Papier, carton.',
    groupe_227 = '227. Energie, génie climatique  (y compris énergie nucléaire, thermique, hydraulique ; utilités ; froid, climatisation, chauffage).',
    groupe_230 = '230. Spécialités pluritechnologiques. Génie civil, construction, bois.',
    groupe_231 = '231. Mines et carrières, génie civil, topographie.',
    groupe_232 = '232. Bâtiment : construction et couverture.',
    groupe_233 = '233. Bâtiment : finitions.',
    groupe_234 = "234. Travail du bois et de l'ameublement.",
    groupe_240 = '240. Spécialités pluritechnologiques Matériaux souples.',
    groupe_241 = '241. Textile.',
    groupe_242 = '242. Habillement (y compris mode, couture).',
    groupe_243 = '243. Cuirs et peaux.',
    groupe_250 = '250. Spécialités pluritechnologiques Mécanique-électricité (y compris maintenance mécano-électrique).',
    groupe_251 = '251. Mécanique générale et de précision, usinage.',
    groupe_252 = '252. Moteurs et mécanique auto.',
    groupe_253 = '253. Mécanique aéronautique et spatiale.',
    groupe_254 = "254. Structures métalliques (y compris soudure, carrosserie, coque de bateau, cellule d'avion).",
    groupe_255 = '255. Electricité, électronique (non compris automatismes, productique).',
    groupe_300 = '300. Spécialités plurivalentes des services.',
    groupe_310 = '310. Spécialités plurivalentes des échanges et de la gestion (y compris administration générale des entreprises et des collectivités).',
    groupe_311 = '311. Transport, manutention, magasinage.',
    groupe_312 = '312. Commerce, vente.',
    groupe_313 = '313. Finance, banques, assurances.',
    groupe_314 = '314. Comptabilité, gestion.',
    groupe_315 = "315. Ressources humaines, gestion du personnel, gestion de l'emploi.",
    groupe_320 = '320. Spécialités plurivalentes de la communication.',
    groupe_321 = '321. Journalisme et communication (y compris communication graphique et publicité).',
    groupe_322 = "322. Techniques de l'imprimerie et de l'édition.",
    groupe_323 = "323. Techniques de l'image et du son, métiers connexes du spectacle.",
    groupe_324 = '324. Secrétariat, bureautique.',
    groupe_325 = '325. Documentation, bibliothèques, administration des données.',
    groupe_326 = "326. Informatique, traitement de l'information, réseau de transmission des données.",
    groupe_330 = '330. Spécialités plurivalentes sanitaires et sociales.',
    groupe_331 = '331. Santé.',
    groupe_332 = '332. Travail social.',
    groupe_333 = '333. Enseignement formation.',
    groupe_334 = '334. Accueil, hôtellerie, tourisme.',
    groupe_335 = '335. Animation culturelle, sportive et de loisirs.',
    groupe_336 = '336. Coiffure, esthétique et autres spécialités des services aux personnes.',
    groupe_340 = '340. Spécialités plurivalentes des services à la collectivité.',
    groupe_341 = '341. Aménagement du territoire, développement, urbanisme.',
    groupe_342 = '342. Protection et développement du patrimoine.',
    groupe_343 = "343. Nettoyage, assainissement, protection de l'environnement.",
    groupe_344 = '344. Sécurité des biens et des personnes, police, surveillance (y compris hygiène et sécurité).',
    groupe_345 = '345. Application des droits et statuts des personnes.',
    groupe_346 = '346. Spécialités militaires.',
    groupe_410 = '410. Spécialités concernant plusieurs capacités.',
    groupe_411 = '411. Pratiques sportives (y compris arts martiaux).',
    groupe_412 = '412. Développement des capacités mentales et apprentissage de base.',
    groupe_413 = '413. Développement des capacités comportementales et relationnelles.',
    groupe_414 = "414. Développement des capacités individuelles d'organisation.",
    groupe_415 = "415. Développement des capacités d'orientation, d'insertion ou de réinsertion sociales et professionnelles.",
    groupe_421 = '421. Jeux et activités spécifiques de loisirs.',
    groupe_422 = '422. Economie et activités domestiques.',
    groupe_423 = '423. Vie familiale, vie sociale et autres formations au développement personnel.',
    aucun = 'aucun'


class groupe_specialites_formation(Variable):
    value_type = Enum
    possible_values = GroupeSpecialitesFormation
    default_value = GroupeSpecialitesFormation.aucun
    entity = Individu
    label = 'Domaine de spécialités en matière de formation'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period
    reference = [
        'https://www.legifrance.gouv.fr/codes/article_lc/LEGIARTI000006526701',
        'https://www.insee.fr/fr/statistiques/fichier/2569957/fqp03_nsf-1.pdf'
        ]


class DomaineSpecialitesFormation(Enum):
    domaine_10 = '10. Formations générales'
    domaine_11 = '11. Mathématiques et sciences'
    domaine_12 = '12. Sciences humaines et droit'
    domaine_13 = '13. Lettres et arts'
    domaine_20 = '20. Spécialités pluritechnologiques de la production'
    domaine_21 = '21. Agriculture, pêche, forêt et espaces verts'
    domaine_22 = '22. Transformations'
    domaine_23 = '23. Génie civil, construction, bois'
    domaine_24 = '24. Matériaux souples'
    domaine_25 = '25. Mécanique, électricité, électronique'
    domaine_30 = '30. Spécialités plurivalentes des services'
    domaine_31 = '31. Echanges et gestion'
    domaine_32 = '32. Communication et information'
    domaine_33 = '33. Services aux personnes'
    domaine_34 = '34. Services à la collectivité'
    domaine_41 = '41. Domaines des capacités individuelles'
    domaine_42 = '42. Domaines des activités quotidiennes et de loisirs'
    aucun = 'aucun'


class domaine_specialites_formation(Variable):
    value_type = Enum
    possible_values = DomaineSpecialitesFormation
    default_value = DomaineSpecialitesFormation.aucun
    entity = Individu
    label = 'Domaine de spécialités en matière de formation'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period
    reference = [
        'https://www.legifrance.gouv.fr/codes/article_lc/LEGIARTI000006526701',
        'https://www.insee.fr/fr/statistiques/fichier/2569957/fqp03_nsf-1.pdf'
        ]

    def formula(individu, period):
        groupe_specialites_formation = individu('groupe_specialites_formation', period).decode_to_str()
        domaines = []
        for groupe in groupe_specialites_formation:
            domaine_number_search = re.search(r'groupe_(\d{2})\d', groupe)
            if domaine_number_search:
                domaines.append(f'domaine_{domaine_number_search.group(1)}')
            else:
                domaines.append('aucun')

        return DomaineSpecialitesFormation.encode(asarray(domaines))



#! /usr/bin/env python
'''Open a variable in the DGFIP website in a browser.'''


import argparse
import logging
import os
import re
import sys
import webbrowser


app_name = os.path.splitext(os.path.basename(__file__))[0]
log = logging.getLogger(app_name)

DEFAULT_YEAR = 2013


def inspect_dgfip_variable(variable, year, browser_name):
    # Attention les liens changent entre la version simplifiée
    matched = re.match('^f[1-8][a-z]{2}', variable)
    assert matched is not None, 'Invalid variable: {}'.format(variable)
    section_number = variable[1]
    case = variable[2:4].upper()
    log.info('section %s, case %s', section_number, case)
    url_base = 'http://www3.impots.gouv.fr/simulateur/calcul_impot/' + str(year + 1) + '/aides/'
    url_section = {
        '2': 'capitaux_mobiliers.htm',
        '3': 'gains_c.htm',
        '4': 'fonciers.htm',
        # '5': "charges_s.htm",
        '6': 'charges.htm',
        '7': 'reductions.htm',
        '8': 'autres_imputations.htm',
        }.get(section_number)
    assert url_section is not None, 'Unhandled section number: {}'.format(section_number)
    url = url_base + url_section
    if section_number not in ('3', '4'):
        url += '#' + case

    browser = webbrowser.get(browser_name)
    browser.open_new_tab(url)


def main():
    parser = argparse.ArgumentParser(description = __doc__)
    parser.add_argument('variable_name', help = 'Name of the variable to inspect. Example: "f7wr"')
    parser.add_argument('--browser', dest = 'browser_name', default = 'chromium', help = 'Open links in this browser')
    parser.add_argument('--min-year', default = None, help = 'Year to start from', type = int)
    parser.add_argument('--max-year', default = None, help = 'Year to stop to', type = int)
    parser.add_argument('-v', '--verbose', action = 'store_true', default = False, help = 'Increase output verbosity')
    parser.add_argument('--year', default = None, help = 'Inspect the variable for the given year', type = int)
    args = parser.parse_args()
    logging.basicConfig(level = logging.DEBUG if args.verbose else logging.WARNING, stream = sys.stdout)

    if args.year and (args.min_year or args.max_year):
        parser.error('year and {min,max}-year arguments cannot be used together')

    if args.min_year or args.max_year:
        if args.min_year and args.max_year:
            for year in range(args.max_year, args.min_year, -1):
                try:
                    inspect_dgfip_variable(args.variable_name, year, args.browser_name)
                except ValueError:
                    log.error('Variable "%s" not found', args.variable_name)
        else:
            parser.error('Please give min and max year')
    else:
        year = args.year or DEFAULT_YEAR
        try:
            inspect_dgfip_variable(args.variable_name, year, args.browser_name)
        except ValueError:
            log.error('Variable "%s" not found', args.variable_name)


if __name__ == '__main__':
    sys.exit(main())



from openfisca_core.model_api import not_, select, where, Variable, MONTH, set_input_divide_by_period, set_input_dispatch_by_period
from openfisca_france.model.base import Famille, Individu, TypesStatutMarital
from openfisca_france.model.prestations.education import TypesScolarite, StatutsEtablissementScolaire
from openfisca_france.model.prestations.education import TypesClasse


class bourse_criteres_sociaux(Variable):
    value_type = float
    entity = Individu
    reference = [
        'Circulaire ESRS2013435C - Annexe 3 - Conditions de ressources et points de charge / 1 - Conditions de ressources',
        'https://www.education.gouv.fr/bo/20/Hebdo25/ESRS2013435C.htm'
        ]
    label = "Montant de la bourse sur critères sociaux (BCS) de l'enseignement supérieur perçue"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        montants = parameters(period).prestations_sociales.aides_jeunes.bourses.bourses_enseignement_superieur.criteres_sociaux.montants
        echelon = individu('bourse_criteres_sociaux_echelon', period)
        return montants.calc(echelon)


class bourse_criteres_sociaux_eligibilite_etude(Variable):
    value_type = bool
    entity = Individu
    reference = [
        "Circulaire ESRS2013435C - Annexe 1 - Conditions d'études",
        'https://www.education.gouv.fr/bo/20/Hebdo25/ESRS2013435C.htm'
        ]
    label = "Satisfaction des critères d'étude pour la bourse sur critères sociaux de l'enseignement supérieur"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period):
        enseignement_superieur = individu('scolarite', period) == TypesScolarite.enseignement_superieur

        temps_plein = not_(individu('alternant', period))

        etablissement = individu('statuts_etablissement_scolaire', period)
        etablissement_eligible = (etablissement == StatutsEtablissementScolaire.public) + (etablissement == StatutsEtablissementScolaire.prive_sous_contrat)

        annee_etude_individus = individu('annee_etude', period)
        annees_etude_doctorat = [TypesClasse.doctorat_1, TypesClasse.doctorat_2, TypesClasse.doctorat_3]
        doctorant = sum([annee_etude_individus == annee_doctorat for annee_doctorat in annees_etude_doctorat])

        return enseignement_superieur * temps_plein * etablissement_eligible * not_(doctorant)


class bourse_criteres_sociaux_eligibilite_nationalite(Variable):
    value_type = bool
    entity = Individu
    reference = [
        'Circulaire ESRS2013435C - 2.3.A',
        'https://www.education.gouv.fr/bo/20/Hebdo25/ESRS2013435C.htm'
        ]
    label = "Satisfaction des critères de nationalité pour la bourse sur critères sociaux de l'enseignement supérieur"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula_2004_07_21(individu, period, parameters):
        '''
        Reference: https://www.education.gouv.fr/bo/2004/30/MENS0401499C.htm
        '''
        ressortissant_eee = individu('ressortissant_eee', period)

        nationalite = individu('nationalite', period)
        ressortissant_pays_eligible = sum([nationalite == str.encode(etat) for etat in parameters(period).prestations_sociales.aides_jeunes.bourses.bourses_enseignement_superieur.criteres_sociaux.nationalites_hors_eee])  # TOOPTIMIZE: string encoding into bytes array should be done at load time

        return ressortissant_eee + ressortissant_pays_eligible

    def formula_2003_04_23(individu, period):
        '''
        Reference: https://www.education.gouv.fr/bo/2003/18/MENS0300894C.htm
        '''
        return individu('resident_ue', period)


class bourse_criteres_sociaux_nombre_enfants_parent_etudiant(Variable):
    value_type = int
    entity = Famille
    reference = [
        "Circulaire ESRS2013435C - Annexe 2 - Critères d'attribution / 1 - Conditions d'âge",
        'https://www.education.gouv.fr/bo/20/Hebdo25/ESRS2013435C.htm'
        ]
    label = "Nombre d'enfants de l'étudiant pour le calcul de la bourse sur critères sociaux de l'enseignement supérieur"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(famille, period):
        return famille.nb_persons(Famille.ENFANT)


class bourse_criteres_sociaux_eligibilite_age(Variable):
    value_type = bool
    entity = Individu
    reference = [
        "Circulaire ESRS2013435C - Annexe 2 - Critères d'attribution / 1 - Conditions d'âge",
        'https://www.education.gouv.fr/bo/20/Hebdo25/ESRS2013435C.htm'
        ]
    label = "Satisfaction des critères d'âge pour la bourse sur critères sociaux de l'enseignement supérieur"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period, parameters):
        age = individu('age', period)
        nb_enf = individu.famille('bourse_criteres_sociaux_nombre_enfants_parent_etudiant', period)
        age_maximum = parameters(period).prestations_sociales.aides_jeunes.bourses.bourses_enseignement_superieur.criteres_sociaux.age_maximum
        handicap = individu('handicap', period)

        return (age <= (age_maximum + nb_enf)) + (handicap)


class bourse_criteres_sociaux_eligibilite(Variable):
    value_type = bool
    entity = Individu
    reference = [
        "Circulaire ESRS2013435C - Annexe 2 - Critères d'attribution",
        'https://www.education.gouv.fr/bo/20/Hebdo25/ESRS2013435C.htm'
        ]
    label = "Éligibilité aux bourses sur critères sociaux de l'enseignement supérieur"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period, parameters):
        eligibilite_age = individu('bourse_criteres_sociaux_eligibilite_age', period)
        eligibilite_etude = individu('bourse_criteres_sociaux_eligibilite_etude', period)
        eligibilite_nationalite = individu('bourse_criteres_sociaux_eligibilite_nationalite', period)

        return eligibilite_age * eligibilite_etude * eligibilite_nationalite


class bourse_criteres_sociaux_base_ressources(Variable):
    value_type = float
    entity = Individu
    reference = [
        'Circulaire ESRS2013435C - Annexe 3 - Conditions de ressources et points de charge / 1 - Conditions de ressources',
        'https://www.education.gouv.fr/bo/20/Hebdo25/ESRS2013435C.htm'
        ]
    label = "Ressources prise en compte pour la bourse sur critères sociaux de l'enseignement supérieur"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period):
        autonome = individu('bourse_criteres_sociaux_etudiant_autonome', period)
        ressources_autonome = individu('bourse_criteres_sociaux_base_ressources_etudiant_autonome', period)
        ressources_parentales = individu('bourse_criteres_sociaux_base_ressources_parentale', period)
        return where(autonome, ressources_autonome, ressources_parentales)


class bourse_criteres_sociaux_base_ressources_parentale(Variable):
    value_type = float
    entity = Individu
    reference = [
        'Circulaire ESRS2013435C - Annexe 3 - Conditions de ressources et points de charge / 1 - Conditions de ressources',
        'https://www.education.gouv.fr/bo/20/Hebdo25/ESRS2013435C.htm'
        ]
    label = "Ressources parentales prises en compte pour la bourse sur critères sociaux de l'enseignement supérieur"
    definition_period = MONTH
    set_input = set_input_divide_by_period


class bourse_criteres_sociaux_base_ressources_etudiant_autonome(Variable):
    value_type = float
    entity = Individu
    reference = [
        'Circulaire ESRS2013435C - Annexe 3 - Conditions de ressources et points de charge / 1 - Conditions de ressources / 1.2.2 - Relatives aux revenus',
        'https://www.education.gouv.fr/bo/20/Hebdo25/ESRS2013435C.htm'
        ]
    label = "Ressources de l'étudiant pour le barème pour la bourse sur critères sociaux de l'enseignement supérieur"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period):
        return individu.foyer_fiscal('rbg', period.n_2)


class bourse_criteres_sociaux_etudiant_autonome_ressource_mensuelle(Variable):
    value_type = float
    entity = Individu
    reference = [
        'Circulaire ESRS2013435C - Annexe 3 - Conditions de ressources et points de charge / 1 - Conditions de ressources / 1.2.2 - Relatives aux revenus',
        'https://www.education.gouv.fr/bo/20/Hebdo25/ESRS2013435C.htm'
        ]
    label = "Ressources mensuelle prise en compte pour déterminer l'autonomie financière de l'étudiant pour la bourse sur critères sociaux de l'enseignement supérieur"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period):
        return individu.famille('rsa_base_ressources', period)


class bourse_criteres_sociaux_etudiant_autonome(Variable):
    value_type = bool
    entity = Individu
    reference = [
        'Circulaire ESRS2013435C - Annexe 3 - Conditions de ressources et points de charge / 1 - Conditions de ressources / 1.2.2 - Relatives aux revenus',
        'https://www.education.gouv.fr/bo/20/Hebdo25/ESRS2013435C.htm'
        ]
    label = "Indicatrice de la satisfaction des critères d'autonomie pour l'étudiant dans le cadre de l'évaluation de la bourse sur critères sociaux"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period, parameters):
        statut_marital = individu('statut_marital', period)
        en_couple = (statut_marital == TypesStatutMarital.marie) + (statut_marital == TypesStatutMarital.pacse)

        ressources = individu('bourse_criteres_sociaux_etudiant_autonome_ressource_mensuelle', period)
        legislation = parameters(period)

        smic_mensuel_brut = legislation.marche_travail.salaire_minimum.smic.smic_b_horaire * legislation.marche_travail.salaire_minimum.smic.nb_heures_travail_mensuel
        smic_mensuel_net = smic_mensuel_brut * 8.11 / 10.25
        seuil_ressources = legislation.prestations_sociales.aides_jeunes.bourses.bourses_enseignement_superieur.criteres_sociaux.seuil_ressources_etudiant_autonome * smic_mensuel_net
        eligible_couple = en_couple * (seuil_ressources <= ressources)

        is_parent = individu.has_role(Famille.PARENT)
        propre_declaration_fiscale = not_(individu('enfant_a_charge', period.this_year))
        avec_des_enfants = individu.famille('bourse_criteres_sociaux_nombre_enfants_parent_etudiant', period) > 0

        eligible_etudiant_parent = is_parent * propre_declaration_fiscale * avec_des_enfants

        orphelin = individu('orphelin', period)

        return eligible_couple + eligible_etudiant_parent + orphelin


class bourse_criteres_sociaux_points_de_charge(Variable):
    entity = Individu
    value_type = int
    reference = [
        "Circulaire ESRS2013435C - Annexe 3 - Conditions de ressources et points de charge / 2 - Points de charge à prendre en considération pour l'attribution d'une bourse sur critères sociaux",
        'https://www.education.gouv.fr/bo/20/Hebdo25/ESRS2013435C.htm'
        ]
    label = "Nombre de points de charge pour la bourse sur critères sociaux de l'enseignement supérieur"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period):
        pts_distance = individu('bourse_criteres_sociaux_points_de_charge_distance_domicile_familial', period)
        pts_charges = individu('bourse_criteres_sociaux_points_de_charge_charges_familiale', period)
        return pts_distance + pts_charges


class bourse_criteres_sociaux_points_de_charge_distance_domicile_familial(Variable):
    entity = Individu
    value_type = int
    reference = [
        "Circulaire ESRS2013435C - Annexe 3 - Conditions de ressources et points de charge / 2 - Points de charge à prendre en considération pour l'attribution d'une bourse sur critères sociaux / 2.1 - Les charges de l'étudiant",
        'https://www.education.gouv.fr/bo/20/Hebdo25/ESRS2013435C.htm'
        ]
    label = "Distance entre le lieu d'étude et le domicile familial pour le calcul la bourse sur critères sociaux de l'enseignement supérieur"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period, parameters):
        distance = individu('bourse_criteres_sociaux_distance_domicile_familial', period)
        bareme = parameters(period).prestations_sociales.aides_jeunes.bourses.bourses_enseignement_superieur.criteres_sociaux.points_de_charge.distance_domicile_familial
        return bareme.calc(distance)


class bourse_criteres_sociaux_distance_domicile_familial(Variable):
    entity = Individu
    value_type = int
    reference = [
        "Circulaire ESRS2013435C - Annexe 3 - Conditions de ressources et points de charge / 2 - Points de charge à prendre en considération pour l'attribution d'une bourse sur critères sociaux / 2.1 - Les charges de l'étudiant",
        'https://www.education.gouv.fr/bo/20/Hebdo25/ESRS2013435C.htm'
        ]
    label = "Distance en kilomètres entre le lieu d'étude et le domicile familial"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class bourse_criteres_sociaux_nombre_enfants_a_charge(Variable):
    entity = Famille
    value_type = int
    reference = [
        "Circulaire ESRS2013435C - Annexe 3 - Conditions de ressources et points de charge / 2 - Points de charge à prendre en considération pour l'attribution d'une bourse sur critères sociaux / 2.2 - Les charges de la famille",
        'https://www.education.gouv.fr/bo/20/Hebdo25/ESRS2013435C.htm'
        ]
    label = "Nombre total d'enfants à la charge de la famille pour le calcul de la bourse sur critères sociaux de l'enseignement supérieur"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class bourse_criteres_sociaux_nombre_enfants_a_charge_dans_enseignement_superieur(Variable):
    entity = Famille
    value_type = int
    reference = [
        "Circulaire ESRS2013435C - Annexe 3 - Conditions de ressources et points de charge / 2 - Points de charge à prendre en considération pour l'attribution d'une bourse sur critères sociaux / 2.2 - Les charges de la famille",
        'https://www.education.gouv.fr/bo/20/Hebdo25/ESRS2013435C.htm'
        ]
    label = "Nombre total d'enfants à la charge de la famille et étudiants dans l'enseignement supérieur pour le calcul la bourse sur critères sociaux de l'enseignement supérieur"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class bourse_criteres_sociaux_points_de_charge_charges_familiale(Variable):
    entity = Individu
    value_type = int
    reference = [
        "Circulaire ESRS2013435C - Annexe 3 - Conditions de ressources et points de charge / 2 - Points de charge à prendre en considération pour l'attribution d'une bourse sur critères sociaux / 2.2 - Les charges de la famille",
        'https://www.education.gouv.fr/bo/20/Hebdo25/ESRS2013435C.htm'
        ]
    label = "Points de charge associés aux charges de la famille pour le alcul la bourse sur critères sociaux de l'enseignement supérieur"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period, parameters):
        nb_enf = individu.famille('bourse_criteres_sociaux_nombre_enfants_a_charge', period)
        nb_enf_ens_sup = individu.famille('bourse_criteres_sociaux_nombre_enfants_a_charge_dans_enseignement_superieur', period)

        nb_enf_hors_ens_sup = nb_enf - nb_enf_ens_sup
        nb_autre_enf_ens_sup = nb_enf_ens_sup - 1

        pts = parameters(period).prestations_sociales.aides_jeunes.bourses.bourses_enseignement_superieur.criteres_sociaux.points_de_charge.charges_familiales
        pts_enf = (nb_enf_hors_ens_sup > 0) * nb_enf_hors_ens_sup * pts.par_enfant_a_charge
        pts_ens_sup = (nb_autre_enf_ens_sup > 0) * nb_autre_enf_ens_sup * pts.par_enfant_etudiant
        return pts_enf + pts_ens_sup


class bourse_criteres_sociaux_echelon(Variable):
    entity = Individu
    value_type = int
    reference = [
        "Circulaire ESRS2013435C - Annexe 7 - Taux et cumul de la bourse d'enseignement supérieur sur critères sociaux / 1 - Les taux de bourse d'enseignement supérieur sur critères sociaux",
        'https://www.education.gouv.fr/bo/20/Hebdo25/ESRS2013435C.htm',
        "Arrêté du 22 juillet 2020 fixant les plafonds de ressources relatifs aux bourses d'enseignement supérieur du ministère de l'enseignement supérieur, de la recherche et de l'innovation pour l'année universitaire 2020-2021",
        'https://www.legifrance.gouv.fr/eli/arrete/2020/7/22/ESRS2016543A/jo/texte'
        ]
    label = "Échelon de la bourse sur critères sociaux de l'enseignement supérieur en prenant uniquement en compte les critères de ressources et de points de charge"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period, parameters):
        points_de_charge = individu('bourse_criteres_sociaux_points_de_charge', period)
        baremes = parameters(period).prestations_sociales.aides_jeunes.bourses.bourses_enseignement_superieur.criteres_sociaux.plafond_ressources
        plafond_echelon_0bis = baremes.echelon_0bis.calc(points_de_charge)
        plafond_echelon_1 = baremes.echelon_1.calc(points_de_charge)
        plafond_echelon_2 = baremes.echelon_2.calc(points_de_charge)
        plafond_echelon_3 = baremes.echelon_3.calc(points_de_charge)
        plafond_echelon_4 = baremes.echelon_4.calc(points_de_charge)
        plafond_echelon_5 = baremes.echelon_5.calc(points_de_charge)
        plafond_echelon_6 = baremes.echelon_6.calc(points_de_charge)
        plafond_echelon_7 = baremes.echelon_7.calc(points_de_charge)

        base_ressources = individu('bourse_criteres_sociaux_base_ressources', period)
        echelon = select(
            [
                base_ressources <= plafond_echelon_7,
                base_ressources <= plafond_echelon_6,
                base_ressources <= plafond_echelon_5,
                base_ressources <= plafond_echelon_4,
                base_ressources <= plafond_echelon_3,
                base_ressources <= plafond_echelon_2,
                base_ressources <= plafond_echelon_1,
                base_ressources <= plafond_echelon_0bis,
                ],
            [7, 6, 5, 4, 3, 2, 1, 0], default=-1)

        eligible = individu('bourse_criteres_sociaux_eligibilite', period)
        return where(eligible, echelon, -1)



import datetime

from .cache import tax_benefit_system


def test_parameters():
    parameters = tax_benefit_system.parameters
    assert parameters is not None
    for year in range(2006, datetime.date.today().year + 1):
        parameters_at_instant = tax_benefit_system.get_parameters_at_instant(year)
        assert parameters_at_instant is not None



from openfisca_core.rates import average_rate, marginal_rate

from openfisca_france.scenarios import init_single_entity

from .cache import tax_benefit_system


def test_average_tax_rate():
    year = 2013
    simulation = init_single_entity(tax_benefit_system.new_scenario(),
        axes = [[
            dict(
                count = 100,
                name = 'salaire_imposable',
                max = 24000,
                min = 0,
                ),
            ]],
        period = year,
        parent1 = dict(age = 40),
        ).new_simulation()
    assert (average_rate(
        target = simulation.calculate('revenu_disponible', period = year),
        varying = simulation.calculate('revenu_disponible', period = year),
        ) == 0).all()


def test_marginal_tax_rate():
    year = 2013
    simulation = init_single_entity(tax_benefit_system.new_scenario(),
        axes = [[
            dict(
                count = 10000,
                name = 'salaire_imposable',
                max = 1000000,
                min = 0,
                ),
            ]],
        period = year,
        parent1 = dict(age = 40),
        ).new_simulation()
    assert (marginal_rate(
        target = simulation.calculate('revenu_disponible', period = year),
        varying = simulation.calculate('revenu_disponible', period = year),
        ) == 0).all()


if __name__ == '__main__':
    import logging
    import sys
    logging.basicConfig(level = logging.ERROR, stream = sys.stdout)
    test_marginal_tax_rate()
    test_average_tax_rate()



from openfisca_france.model.base import *

from numpy import timedelta64

from openfisca_france.model.prestations.education import StatutsEtablissementScolaire


class aide_mobilite_internationale_eligibilite(Variable):
    value_type = bool
    entity = Individu
    label = "Éligibilite à l'aide à la mobilité internationale (AMI)"
    definition_period = MONTH

    def formula(individu, period, parameters):
        '''
        Conditions non modélisées :
        - Éligibilité des boursiers bénéficiaires d'une allocation annuelle (dispositif des aides spécifiques).
        - La formation ou stage à l'étranger s'inscrit dans le cadre du cursus d'études.
        - Au cours de l'ensemble des études supérieures, ne peuvent être cumulés plus 9 mois d'aide à la mobilité internationale (sauf pour les étudiants ayant perçu neuf mensualités de l'aide à la mobilité internationale et dont le séjour à l'étranger a été interrompu en raison de l'épidémie de Covid-19, qui peuvent bénéficier de mensualités supplémentaires dans le cadre d'une mobilité ultérieure, dans la limite de la durée de la mobilité non effectuée.)
        '''
        debut_etudes_etranger = individu('debut_etudes_etranger', period)
        fin_etudes_etranger = individu('fin_etudes_etranger', period)
        duree_etudes_etranger = (fin_etudes_etranger - debut_etudes_etranger).astype('timedelta64[M]')
        eligibilite_duree_min = duree_etudes_etranger >= timedelta64(parameters(period).prestations_sociales.aides_jeunes.mobilite.internationale.duree_sejour.mois_min, 'M')
        eligibilite_duree_max = duree_etudes_etranger <= timedelta64(parameters(period).prestations_sociales.aides_jeunes.mobilite.internationale.duree_sejour.mois_max, 'M')

        statuts_etablissement_scolaire = individu('statuts_etablissement_scolaire', period)
        etablissement_eligible = (statuts_etablissement_scolaire == StatutsEtablissementScolaire.public) + (statuts_etablissement_scolaire == StatutsEtablissementScolaire.prive_sous_contrat)

        bourse_criteres_sociaux_eligibilite = individu('bourse_criteres_sociaux_eligibilite', period)

        return bourse_criteres_sociaux_eligibilite * eligibilite_duree_min * eligibilite_duree_max * etablissement_eligible


class aide_mobilite_internationale(Variable):
    value_type = float
    entity = Individu
    label = "Montant de l'aide à la mobilité internationale (AMI)"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = [
        'Circulaire du 8-6-2020 - Modalités d’attribution des bourses d’enseignement supérieur - Annexe 9 : Aide à la mobilité internationale',
        'https://www.enseignementsup-recherche.gouv.fr/pid20536/bulletin-officiel.html?cid_bo=152353&cbo=1',
        ]

    def formula(individu, period, parameters):
        '''
        Ce calcul ne détermine qu'une éligibilité potentielle et n'ouvre pas de droits : l'aide est accordée par une évaluation de l'établissement d'origine, en fonction de la durée de votre séjour et de certaines spécificités telles que l'éloignement du pays d'accueil, le coût de la vie du pays choisi.
        '''
        return individu('aide_mobilite_internationale_eligibilite', period) * parameters(period).prestations_sociales.aides_jeunes.mobilite.internationale.montant



from numpy import busday_count, datetime64, timedelta64, where

from openfisca_france.model.base import *


class chomeur_longue_duree(Variable):
    cerfa_field = {
        0: '1AI',
        1: '1BI',
        2: '1CI',
        3: '1DI',
        4: '1EI',
        }
    value_type = bool
    entity = Individu
    label = "Demandeur d'emploi inscrit depuis plus d'un an"
    definition_period = YEAR
    end = '2017-12-31'
    # Pour toutes les variables de ce type, les pac3 ne sont plus proposés après 2007


class chomage_brut(Variable):
    value_type = float
    entity = Individu
    label = 'Chômage brut'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    calculate_output = calculate_output_add

    def formula(individu, period):
        # pas de cumul des revenus de remplacement :
        # ARE (demandeur d'emploi) vs. complément ARE (reprise d'activité + droits au chômage)
        complement_are_brut = individu('complement_are_brut', period)
        allocation_retour_emploi = individu('allocation_retour_emploi', period)

        return complement_are_brut + allocation_retour_emploi


class indemnites_chomage_partiel(Variable):
    value_type = float
    entity = Individu
    label = 'Indemnités de chômage partiel'
    definition_period = MONTH
    set_input = set_input_divide_by_period


class allocation_retour_emploi(Variable):
    value_type = float
    entity = Individu
    label = "Allocation chômage d'aide au retour à l'emploi (ARE) mensuelle brute"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    unit = 'currency'
    reference = 'https://www.unedic.org/indemnisation/fiches-thematiques/allocation-daide-au-retour-lemploi-are'

    def formula(individu, period):
        # L'ARE comprend de nombreuses conditions d'éligibilité non modélisées ici.
        activite = individu('activite', period)  # demandeur d'emploi inscrit à Pôle emploi
        allocation_retour_emploi_montant = individu('allocation_retour_emploi_montant', period)

        return (activite == TypesActivite.chomeur) * allocation_retour_emploi_montant


class allocation_retour_emploi_montant(Variable):
    value_type = float
    entity = Individu
    label = "Montant de l'allocation chômage d'aide au retour à l'emploi (ARE) mensuelle brute"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = [
        'https://www.legifrance.gouv.fr/codes/id/LEGISCTA000006178163/',
        'https://www.unedic.org/indemnisation/fiches-thematiques/cumul-allocation-salaire'
        ]

    def formula(individu, period):
        # Attention : ARE simplifiée (modélisation suffisante pour le Complément ARE)
        debut_mois = datetime64(period.start.offset('first-of', 'month'))
        fin_mois = datetime64(period.start.offset('last-of', 'month')) + timedelta64(1, 'D')
        nombre_jours_mois = busday_count(
            debut_mois,
            fin_mois,
            weekmask= '1' * 7
            )

        degressivite_are = individu('degressivite_are', period)
        allocation_journaliere_taux_plein = individu('allocation_retour_emploi_journaliere_taux_plein', period)
        allocation_journaliere = individu('allocation_retour_emploi_journaliere', period)

        return where(
            degressivite_are,
            nombre_jours_mois * allocation_journaliere_taux_plein,
            nombre_jours_mois * allocation_journaliere
            )


class allocation_retour_emploi_journaliere(Variable):
    value_type = float
    entity = Individu
    label = "Allocation chômage d'aide au retour à l'emploi (ARE) brute journalière au sens Pôle Emploi"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = 'https://www.legifrance.gouv.fr/codes/id/LEGISCTA000006178163/'


class allocation_retour_emploi_journaliere_taux_plein(Variable):
    value_type = float
    entity = Individu
    label = 'Allocation journalière ARE brute taux plein'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period):
        return individu('allocation_retour_emploi_journaliere', period)


class allocation_travailleur_independant(Variable):
    value_type = float
    entity = Individu
    label = 'Allocation des travailleurs indépendants (ATI)'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = 'https://www.legifrance.gouv.fr/codes/id/LEGISCTA000037388330/'
    documentation = '''
    Indemnisation de Pôle emploi en vigueur à partir du 1er novembre 2019 à destination
    des travailleurs non salariés indépendants contraints de mettre fin à leur activité.
    '''


class are_salaire_journalier_reference(Variable):
    value_type = float
    entity = Individu
    label = 'Salaire journalier de référence (SJR) au sens des allocations chômage'
    definition_period = MONTH
    reference = 'https://www.legifrance.gouv.fr/loda/article_lc/LEGIARTI000043628391'
    set_input = set_input_divide_by_period


class degressivite_are(Variable):
    value_type = bool
    entity = Individu
    label = "L'individu est soumis à la dégressivité de l'allocation d'Aide au Retour à l'Emploi (ARE)"
    definition_period = MONTH
    reference = [
        'https://www.legifrance.gouv.fr/jorf/id/JORFTEXT000044345334',
        'https://www.legifrance.gouv.fr/loda/id/JORFTEXT000041798325/'
        ]
    set_input = set_input_divide_by_period



from openfisca_core.periods import Period

from openfisca_france.model.base import *


# TODO: 5QL


# Nomenclature :
# première lettre :
#    e : auto-entrepreneur
#    m : micro entreprise, déclaratif spécial
#    n : bénéfice réel sans CGA
#    a : bénéfice réel avec CGA ou viseur
#    f : forfait
#    c : déclaration contrôlée)
# trois lettres suivantes, catégorie du revenu :
#    rag : agricole
#    bic : industriel et commercial pro
#    bnc : non commercial pro
#    acc : industriel et commercial non pro
#    ncn : non commercial non pro
# après l'underscore : abbréviation du label de la case

# (f5qm, f5rm )
class f5qm(Variable):
    cerfa_field = {0: '5QM',
        1: '5RM',
                   }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Agents généraux d’assurances: indemnités de cessation d’activité'
    definition_period = YEAR


# """
# Revenus des professions non salariées
# """

# (f5nv, f5ov, f5pv)
class ppe_du_ns(Variable):
    cerfa_field = {0: '5NV',
        1: '5OV',
        2: '5PV',
                   }
    value_type = int
    entity = Individu
    label = "Prime pour l'emploi des non-salariés: nombre de jours travaillés dans l'année"
    end = '2014-12-31'
    definition_period = YEAR


# (f5nw, f5ow, f5pw)
class ppe_tp_ns(Variable):
    cerfa_field = {0: '5NW',
        1: '5OW',
        2: '5PW',
                   }
    value_type = bool
    entity = Individu
    label = "Prime pour l'emploi des non-salariés: indicateur de travail à temps plein sur l'année entière"
    end = '2014-12-31'
    definition_period = YEAR


# (f5hn, f5in, f5jn))
class frag_exon(Variable):
    cerfa_field = {0: '5HN',
        1: '5IN',
        2: '5JN', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus agricoles exonérés (régime du forfait)'
    # start_date = date(2007, 1, 1)
    end = '2015-12-31'
    definition_period = YEAR


# (f5ha, f5ia, f5ja))
class arag_info(Variable):
    cerfa_field = {0: '5HA',
        1: '5IA',
        2: '5JA', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus agricoles: revenus nets de la cession ou concession de brevets et assimilés'
    # start_date = date(2019, 1, 1)
    definition_period = YEAR


# (f5hd, f5id, f5jd))
class coupe_bois(Variable):
    cerfa_field = {0: '5HD',
        1: '5ID',
        2: '5JD', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus forfaitaire provenant des coupes de bois'
    # start_date = date(2016, 1, 1)
    definition_period = YEAR


# (f5xa, f5ya, f5za))
class mrag_exon(Variable):
    cerfa_field = {0: '5XA',
        1: '5YA',
        2: '5ZA', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus agricoles exonérés (régime micro BA)'
    # start_date = date(2016, 1, 1)
    definition_period = YEAR


# (f5ho, f5io, f5jo))
class frag_impo(Variable):
    cerfa_field = {0: '5HO',
        1: '5IO',
        2: '5JO', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus agricoles imposables (régime du forfait)'
    # start_date = date(2007, 1, 1)
    end = '2015-12-31'
    definition_period = YEAR


# (f5xb, f5yb, f5zb))
class mrag_impo(Variable):
    cerfa_field = {0: '5XB',
        1: '5YB',
        2: '5ZB', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus agricoles imposables (régime micro BA)'
    # start_date = date(2016, 1, 1)
    definition_period = YEAR


# (f5xc, f5yc, f5zc)
class date_creation(Variable):
    cerfa_field = {0: '5XC',
        1: '5YC',
        2: '5ZC',
                   }
    value_type = int
    entity = Individu
    label = 'Revenus agricoles : Année de création de l’activité'
    # start_date = date(2016, 1, 1)
    end = '2017-12-31'
    definition_period = YEAR


# (f5xd, f5yd, f5zd))
class frag_impo_n2(Variable):
    cerfa_field = {0: '5XD',
        1: '5YD',
        2: '5ZD', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus agricoles, forfait ou microBA année n-2'
    # start_date = date(2016, 1, 1)
    end = '2017-12-31'
    definition_period = YEAR


# (f5xe, f5ye, f5ze))
class frag_impo_n1(Variable):
    cerfa_field = {0: '5XE',
        1: '5YE',
        2: '5ZE', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus agricoles, forfait ou microBA année n-1'
    # start_date = date(2016, 1, 1)
    end = '2017-12-31'
    definition_period = YEAR


# (f5xf, f5yf, f5zf))
class arag_impo_n2(Variable):
    cerfa_field = {0: '5XF',
        1: '5YF',
        2: '5ZF', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus agricoles, régime du bénéfice réel année n-2'
    # start_date = date(2016, 1, 1)
    end = '2017-12-31'
    definition_period = YEAR


# (f5xg, f5yg, f5zg))
class arag_impo_n1(Variable):
    cerfa_field = {0: '5XG',
        1: '5YG',
        2: '5ZG', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus agricoles, régime du bénéfice réel année n-1'
    # start_date = date(2016, 1, 1)
    end = '2016-12-31'
    definition_period = YEAR


# (f5hb, f5ib, f5jb))
class arag_exon(Variable):
    cerfa_field = {0: '5HB',
        1: '5IB',
        2: '5JB', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = "Revenus agricoles exonérés yc plus-values (Régime du bénéfice réel, revenus bénéficiant de l'abattement CGA ou viseur), activités exercées en Corse"
    # start_date = date(2007, 1, 1)
    definition_period = YEAR


# (f5hc, f5ic, f5jc))
class arag_impg(Variable):
    cerfa_field = {0: '5HC',
        1: '5IC',
        2: '5JC', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = "Revenus agricoles imposables, cas général moyenne triennale (Régime du bénéfice réel, revenus bénéficiant de l'abattement CGA ou viseur)"
    # start_date = date(2007, 1, 1)
    definition_period = YEAR


# (f5hf, f5if, f5jf))
class arag_defi(Variable):
    cerfa_field = {0: '5HF',
        1: '5IF',
        2: '5JF', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = "Déficits agricoles (Régime du bénéfice réel, revenus bénéficiant de l'abattement CGA ou viseur)"
    # start_date = date(2007, 1, 1)
    definition_period = YEAR


# (f5hh, f5ih, f5jh))
class nrag_exon(Variable):
    cerfa_field = {0: '5HH',
        1: '5IH',
        2: '5JH', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = "Revenus agricoles exonérés yc plus-values (Régime du bénéfice réel, revenus ne bénéficiant pas de l'abattement CGA ou viseur), activités exercées en Corse"
    # start_date = date(2007, 1, 1)
    definition_period = YEAR


# (f5hi, f5ii, f5ji))
class nrag_impg(Variable):
    cerfa_field = {0: '5HI',
        1: '5II',
        2: '5JI', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = "Revenus agricoles imposables, cas général moyenne triennale (Régime du bénéfice réel, revenus ne bénéficiant pas de l'abattement CGA ou viseur)"
    # start_date = date(2007, 1, 1)
    definition_period = YEAR


# (f5hl, f5il, f5jl))
class nrag_defi(Variable):
    cerfa_field = {0: '5HL',
        1: '5IL',
        2: '5JL', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = "Déficits agricoles (Régime du bénéfice réel, revenus ne bénéficiant pas de l'abattement CGA ou viseur)"
    # start_date = date(2007, 1, 1)
    definition_period = YEAR


# (f5hm, f5im, f5jm))
class nrag_ajag(Variable):
    cerfa_field = {0: '5HM',
        1: '5IM',
        2: '5JM', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = "Jeunes agriculteurs, Abattement de 50% ou 100% (Régime du bénéfice réel, revenus bénéficiant de l'abattement CGA ou viseur)"
    # start_date = date(2007, 1, 1)
    definition_period = YEAR


# """
# Autoentrepreneur
# """


# (f5ta, f5ua, f5va))
class ebic_impv(Variable):
    cerfa_field = {0: '5TA',
        1: '5UA',
        2: '5VA', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus industriels et commerciaux professionnels imposables: vente de marchandises et assimilées (régime auto-entrepreneur ayant opté pour le versement libératoire)'
    # start_date = date(2009, 1, 1)
    definition_period = YEAR


# (f5tb, f5ub, f5vb))
class ebic_imps(Variable):
    cerfa_field = {0: '5TB',
        1: '5UB',
        2: '5VB', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus industriels et commerciaux professionnels imposables: prestations de services et locations meublées (régime auto-entrepreneur ayant opté pour le versement libératoire)'
    # start_date = date(2009, 1, 1)
    definition_period = YEAR


# (f5te, f5ue, f5ve))
class ebnc_impo(Variable):
    cerfa_field = {0: '5TE',
        1: '5UE',
        2: '5VE', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus non commerciaux (régime auto-entrepreneur ayant opté pour le versement libératoire)'
    # start_date = date(2009, 1, 1)
    definition_period = YEAR


# (f5kn, f5ln, f5mn))
class mbic_exon(Variable):
    cerfa_field = {0: '5KN',
        1: '5LN',
        2: '5MN', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus industriels et commerciaux professionnels nets exonérés (régime micro entreprise)'
    definition_period = YEAR


# (f5kb, f5lb, f5mb))
class abic_exon(Variable):
    cerfa_field = {0: '5KB',
        1: '5LB',
        2: '5MB', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus industriels et commerciaux nets exonérés yc plus-values avec CGA ou viseur (régime du bénéfice réel)'
    definition_period = YEAR


# (f5kh, f5lh, f5mh))
class nbic_exon(Variable):
    cerfa_field = {0: '5KH',
        1: '5LH',
        2: '5MH', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus industriels et commerciaux nets exonérés yc plus-values sans CGA (régime du bénéfice réel)'
    definition_period = YEAR


# (f5ko, f5lo, f5mo))
class mbic_impv(Variable):
    cerfa_field = {0: '5KO',
        1: '5LO',
        2: '5MO', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus industriels et commerciaux professionnels imposables: vente de marchandises (régime micro entreprise)'
    definition_period = YEAR


# (f5kp, f5lp, f5mp))
class mbic_imps(Variable):
    cerfa_field = {0: '5KP',
        1: '5LP',
        2: '5MP', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus industriels et commerciaux professionnels imposables: prestations de services et locations meublées (régime micro entreprise)'
    definition_period = YEAR


# (f5kc, f5lc, f5mc))
class abic_impn(Variable):
    cerfa_field = {0: '5KC',
        1: '5LC',
        2: '5MC', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus industriels et commerciaux imposables: régime normal ou simplifié avec CGA ou viseur (régime du bénéfice réel)'
    definition_period = YEAR


# (f5kd, f5ld, f5md))
class abic_imps(Variable):
    cerfa_field = {0: '5KD',
        1: '5LD',
        2: '5MD', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus industriels et commerciaux imposables: régime simplifié avec CGA ou viseur (régime du bénéfice réel)'
    end = '2009-12-31'
    definition_period = YEAR


# (f5ki, f5li, f5mi))
class nbic_impn(Variable):
    cerfa_field = {0: '5KI',
        1: '5LI',
        2: '5MI', }

    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus industriels et commerciaux professionnels imposables: régime normal ou simplifié sans CGA (régime du bénéfice réel)'
    definition_period = YEAR


# (f5ui, f5vi, f5wi))
class abic_info(Variable):
    cerfa_field = {0: '5UI',
        1: '5VI',
        2: '5WI', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus industriels et commerciaux professionnels : Revenus nets de la cession ou concession de brevets et assimilés'
    # start_date = date(2019, 1, 1)
    definition_period = YEAR


# """
# Réutilisation cases 2013
# """


# (f5kj, f5lj, f5mj))
class nbic_imps(Variable):
    cerfa_field = {0: '5KJ',
        1: '5LJ',
        2: '5MJ', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus industriels et commerciaux professionnels imposables: régime simplifié sans CGA (régime du bénéfice réel)'
    end = '2009-12-31'
    definition_period = YEAR


# (f5kj, f5lj, f5mj))
# NB cette variable devrait s'appeler 'mbic_mvct' comme c'est une variable de régime micro (mais il
# existe déjà une variable mbic_mvct avec un autre cerfa_fiel (5HU) corresponsant à avant 2012...)
class nbic_mvct(Variable):
    cerfa_field = {0: '5KJ',
        1: '5LJ',
        2: '5MJ', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus industriels et commerciaux professionnels moins-values nettes à court terme : régime micro-entreprise'
    # start_date = date(2012, 1, 1)
    definition_period = YEAR


# (f5kf, f5lf, f5mf))
class abic_defn(Variable):
    cerfa_field = {0: '5KF',
        1: '5LF',
        2: '5MF', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Déficits industriels et commerciaux: régime normal ou simplifié avec CGA ou viseur (régime du bénéfice réel)'
    definition_period = YEAR


# (f5kg, f5lg, f5mg))
# vérif <=2012
class abic_defs(Variable):
    cerfa_field = {0: '5KG',
        1: '5LG',
        2: '5MG', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Déficits industriels et commerciaux: simplifié avec CGA ou viseur (régime du bénéfice réel)'
    end = '2009-12-01'
    definition_period = YEAR


# (f5kl, f5ll, f5ml))
class nbic_defn(Variable):
    cerfa_field = {0: '5KL',
        1: '5LL',
        2: '5ML', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Déficits industriels et commerciaux: régime normal ou simplifié sans CGA (régime du bénéfice réel)'
    definition_period = YEAR


# (f5km, f5lm, f5mm))
class nbic_defs(Variable):
    cerfa_field = {0: '5KM',
        1: '5LM',
        2: '5MM', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Locations déjà soumises aux prélèvements sociaux sans CGA (régime du bénéfice réel)'
    end = '2009-12-31'
    definition_period = YEAR


# (f5ks, f5ls, f5ms))
class nbic_apch(Variable):
    cerfa_field = {0: '5KS',
        1: '5LS',
        2: '5MS', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Artisans pêcheurs : abattement 50% avec CGA ou viseur (régime du bénéfice réel)'
    end = '2017-12-31'
    definition_period = YEAR


# (f5nn, f5on, f5pn))
class macc_exon(Variable):
    cerfa_field = {0: '5NN',
        1: '5ON',
        2: '5PN', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus industriels et commerciaux non professionnels nets exonérés (régime micro entreprise)'
    definition_period = YEAR


# (f5nb, f5ob, f5pb))
class aacc_exon(Variable):
    cerfa_field = {0: '5NB',
        1: '5OB',
        2: '5PB', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus industriels et commerciaux non professionnels exonérés yc plus-values avec CGA ou viseur (régime du bénéfice réel)'
    definition_period = YEAR


# (f5tf, f5uf, f5vf))
class aacc_info(Variable):
    cerfa_field = {0: '5TF',
        1: '5UF',
        2: '5VF', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus industriels et commerciaux non-professionnels : Revenus nets de la cession ou concession de brevets et assimilés'
    # start_date = date(2019, 1, 1)
    definition_period = YEAR


# (f5nh, f5oh, f5ph))
class nacc_exon(Variable):
    cerfa_field = {0: '5NH',
        1: '5OH',
        2: '5PH', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus industriels et commerciaux non professionnels exonérés yc plus-values sans CGA (régime du bénéfice réel)'
    definition_period = YEAR


# (f5no, f5oo, f5po))
class macc_impv(Variable):
    cerfa_field = {0: '5NO',
        1: '5OO',
        2: '5PO', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus industriels et commerciaux non professionnels imposables: vente de marchandises et assimilées (régime micro entreprise)'
    definition_period = YEAR


# (f5np, f5op, f5pp))
class macc_imps(Variable):
    cerfa_field = {0: '5NP',
        1: '5OP',
        2: '5PP', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus industriels et commerciaux non professionnels imposables: prestations de services (régime micro entreprise)'
    definition_period = YEAR


# (f5nc, f5oc, f5pc))
class aacc_impn(Variable):
    cerfa_field = {0: '5NC',
        1: '5OC',
        2: '5PC', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus industriels et commerciaux non professionnels imposables: régime normal ou simplifié avec CGA ou viseur (régime du bénéfice réel)'
    definition_period = YEAR


# (f5nd, f5od, f5pd)) #TODO: avant 2010
class aacc_imps(Variable):
    cerfa_field = {0: '5ND',
        1: '5OD',
        2: '5PD', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Locations meublées non professionnelles (régime micro entreprise)'
    # start_date = date(2011, 1, 1)
    definition_period = YEAR


# (f5nf, f5of, f5pf))
class aacc_defn(Variable):
    cerfa_field = {0: '5NF',
        1: '5OF',
        2: '5PF', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Déficits industriels et commerciaux non professionnels: régime normal ou simplifié avec CGA ou viseur (régime du bénéfice réel)'
    definition_period = YEAR


# (f5ng, f5og, f5pg))
class aacc_gits(Variable):
    cerfa_field = {0: '5NG',
        1: '5OG',
        2: '5PG', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = "Location de gîtes ruraux, chambres d'hôtes et meublés de tourisme (régime micro entreprise)"
    # start_date = date(2011, 1, 1)
    definition_period = YEAR


# (f5ni, f5oi, f5pi))
class nacc_impn(Variable):
    cerfa_field = {0: '5NI',
        1: '5OI',
        2: '5PI', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus industriels et commerciaux non professionnels imposables: régime normal ou simplifié sans CGA (régime du bénéfice réel)'
    definition_period = YEAR


# (f5ng, f5og, f5pg))
class aacc_defs(Variable):
    cerfa_field = {0: '5NG',
        1: '5OG',
        2: '5PG', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Déficits de revenus industriels et commerciaux non professionnels avec CGA (régime simplifié du bénéfice réel)'
    end = '2009-12-31'
    definition_period = YEAR


# (f5nj, f5oj, f5pj)) #TODO: dates 5PJ, 5PG, 5PD, 5OM
class nacc_meup(Variable):
    cerfa_field = {0: '5NJ',
        1: '5OJ',
        2: '5PJ', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Locations meublées non professionnelles: Locations déjà soumises aux prélèvements sociaux (régime micro entreprise)'
    # start_date = date(2012, 1, 1)
    definition_period = YEAR


# (f5nw, f5ow, f5pw))
class nacc_meuc(Variable):
    cerfa_field = {0: '5NW',
        1: '5OW',
        2: '5PW', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Locations meublées non professionnelles: cas général des locations déjà soumises aux prélèvements sociaux (régime micro entreprise)'
    # start_date = (2017,1,1)
    definition_period = YEAR


# (f5nl, f5ol, f5pl))
class nacc_defn(Variable):
    cerfa_field = {0: '5NL',
        1: '5OL',
        2: '5PL', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Déficits industriels et commerciaux non professionnels: régime normal ou simplifié sans CGA (régime du bénéfice réel)'
    definition_period = YEAR


# (f5nm, f5om, f5pm)) #TODO autres 5NM
class nacc_defs(Variable):
    cerfa_field = {0: '5NM',
        1: '5OM',
        2: '5PM', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = "Locations meublées non professionnelles: Gîtes ruraux et chambres d'hôtes déjà soumis aux prélèvements sociaux avec CGA (régime du bénéfice réel)"
    # start_date = date(2012, 1, 1)
    definition_period = YEAR


# (f5ku, f5lu, f5mu))
class mncn_impo(Variable):
    cerfa_field = {0: '5KU',
        1: '5LU',
        2: '5MU', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus non commerciaux non professionnels imposables (régime déclaratif spécial ou micro BNC)'
    definition_period = YEAR


# (f5sn, f5ns, f5os))
class cncn_bene(Variable):
    cerfa_field = {0: '5SN',
        1: '5NS',
        2: '5OS', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus non commerciaux non professionnels imposables sans AA (régime de la déclaration controlée)'
    # start_date = date(2006, 1, 1)
    definition_period = YEAR


# (f5sp, f5nu, f5ou, f5sr))
# pas de f5sr en 2013
class cncn_defi(Variable):
    cerfa_field = {0: '5SP',
        1: '5NU',
        2: '5OU', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Déficits non commerciaux non professionnels sans AA (régime de la déclaration controlée)'
    # start_date = date(2006, 1, 1)
    definition_period = YEAR


# (f5hp, f5ip, f5jp))
class mbnc_exon(Variable):
    cerfa_field = {0: '5HP',
        1: '5IP',
        2: '5JP', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus non commerciaux professionnels nets exonérés (régime déclaratif spécial ou micro BNC)'
    definition_period = YEAR


# (f5qb, f5rb, f5sb))
class abnc_exon(Variable):
    cerfa_field = {0: '5QB',
        1: '5RB',
        2: '5SB', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = "Revenus non commerciaux professionnels exonérés (yc compris plus-values) (régime de la déclaration controlée, revenus bénéficiant de l'abattement association agrée ou viseur)"
    definition_period = YEAR


# (f5qh, f5rh, f5sh))
class nbnc_exon(Variable):
    cerfa_field = {0: '5QH',
        1: '5RH',
        2: '5SH', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = "Revenus non commerciaux professionnels exonérés (yc compris plus-values) (régime de la déclaration controlée, revenus ne bénéficiant pas de l'abattement association agrée)"
    definition_period = YEAR


# (f5hq, f5iq, f5jq))
class mbnc_impo(Variable):
    cerfa_field = {0: '5HQ',
        1: '5IQ',
        2: '5JQ', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus non commerciaux professionnels imposables (régime déclaratif spécial ou micro BNC)'
    definition_period = YEAR


# (f5qa, f5ra, f5sa))
class abnc_info(Variable):
    cerfa_field = {0: '5QA',
        1: '5RA',
        2: '5SA', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus non-commerciaux professionnels imposables: Revenus nets de la cession ou concession de brevets et assimilés (régime de déclaration controlée)'
    # start_date = date(2019, 1, 1)
    definition_period = YEAR


# (f5qc, f5rc, f5sc))
class abnc_impo(Variable):
    cerfa_field = {0: '5QC',
        1: '5RC',
        2: '5SC', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = "Revenus non commerciaux professionnels imposables (régime de la déclaration controlée, revenus bénéficiant de l'abattement association agrée ou viseur)"
    definition_period = YEAR


# (f5qe, f5re, f5se))
class abnc_defi(Variable):
    cerfa_field = {0: '5QE',
        1: '5RE',
        2: '5SE', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = "Déficits non commerciaux professionnels (régime de la déclaration controlée, revenus bénéficiant de l'abattement association agrée ou viseur)"
    definition_period = YEAR


# (f5qi, f5ri, f5si))
class nbnc_impo(Variable):
    cerfa_field = {0: '5QI',
        1: '5RI',
        2: '5SI', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = "Revenus non commerciaux professionnels imposables (régime de la déclaration controlée, revenus ne bénéficiant pas de l'abattement association agrée)"
    definition_period = YEAR


# (f5qk, f5rk, f5sk))
class nbnc_defi(Variable):
    cerfa_field = {0: '5QK',
        1: '5RK',
        2: '5SK', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = "Déficits non commerciaux professionnels (régime de la déclaration controlée, revenus ne bénéficiant pas de l'abattement association agrée)"
    definition_period = YEAR


# (f5hu))
# vérif <=2012
class mbic_mvct(Variable):
    cerfa_field = '5HU'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Moins-values industrielles et commerciales nettes à court terme du foyer (régime micro entreprise)'
    end = '2011-12-31'
    definition_period = YEAR


# (f5iu))
class macc_mvct(Variable):
    cerfa_field = '5IU'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Moins-values industrielles et commerciales non professionnelles nettes à court terme du foyer (régime micro entreprise)'
    definition_period = YEAR


# (f5ju))
class mncn_mvct(Variable):
    cerfa_field = '5JU'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Moins-values non commerciales non professionnelles nettes à court terme du foyer (régime déclaratif spécial ou micro BNC)'
    definition_period = YEAR


# (f5kz, f5lz , f5mz), f5lz , f5mz sont présentent en 2013
class mbnc_mvct(Variable):
    cerfa_field = {0: '5KZ',  # TODO: pb cerfa field
        1: '5LZ',
        2: '5MZ', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Moins-values non commerciales professionnelles nettes à court terme (régime déclaratif spécial ou micro BNC)'
    # start_date = date(2012, 1, 1)
    definition_period = YEAR


# (f5hw, f5iw, f5jw))
class frag_pvct(Variable):
    cerfa_field = {0: '5HW',
        1: '5IW',
        2: '5JW', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Plus-values agricoles  à court terme (régime du forfait)'
    # start_date = date(2007, 1, 1)
    end = '2015-12-31'
    definition_period = YEAR


# (f5kx, f5lx, f5mx))
class mbic_pvct(Variable):
    cerfa_field = {0: '5KX',
        1: '5LX',
        2: '5MX', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Plus-values industrielles et commerciales professionnels imposables: plus-values nettes à court terme (régime micro entreprise)'
    definition_period = YEAR


# (f5nx, f5ox, f5px))
class macc_pvct(Variable):
    cerfa_field = {0: '5NX',
        1: '5OX',
        2: '5PX', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Plus-values industrielles et commerciales non professionnelles imposables: plus-values nettes à court terme (régime micro entreprise)'
    definition_period = YEAR


# (f5hv, f5iv, f5jv))
class mbnc_pvct(Variable):
    cerfa_field = {0: '5HV',
        1: '5IV',
        2: '5JV', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Plus-values non commerciales professionnelles imposables et Plus-values nettes à court terme (régime déclaratif spécial ou micro BNC)'
    definition_period = YEAR


# (f5ky, f5ly, f5my))
class mncn_pvct(Variable):
    cerfa_field = {0: '5KY',
        1: '5LY',
        2: '5MY', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Plus-values non commerciales non professionnelles imposables et plus-values nettes à court terme (régime déclaratif spécial ou micro BNC)'
    definition_period = YEAR


# (f5kr, f5lr, f5mr))
class mbic_mvlt(Variable):
    cerfa_field = {0: '5KR',
        1: '5LR',
        2: '5MR', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Moins-values industrielles et commerciales professionnels à long terme (régime micro entreprise)'
    definition_period = YEAR


# (f5nr, f5or, f5pr))
class macc_mvlt(Variable):
    cerfa_field = {0: '5NR',
        1: '5OR',
        2: '5PR', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Moins-values industrielles et commerciales non professionnelles à long terme (régime micro entreprise)'
    definition_period = YEAR


# (f5kw, f5lw, f5mw))
class mncn_mvlt(Variable):
    cerfa_field = {0: '5KW',
        1: '5LW',
        2: '5MW', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Moins-values non commerciales non professionnelles à long terme (régime déclaratif spécial ou micro BNC)'
    definition_period = YEAR


# (f5hs, f5is, f5js))
class mbnc_mvlt(Variable):
    cerfa_field = {0: '5HS',
        1: '5IS',
        2: '5JS', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Moins-values non commerciales professionnelles à long terme (régime déclaratif spécial ou micro BNC)'
    definition_period = YEAR


# (f5hx, f5ix, f5jx))
class frag_pvce(Variable):
    cerfa_field = {0: '5HX',
        1: '5IX',
        2: '5JX', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Plus-values agricoles de cession (régime du forfait ou MICRO BA) taxables à 16%'
    # start_date = date(2007, 1, 1)
    end = '2016-12-31'
    definition_period = YEAR


# (f5hw, f5iw, f5jw))
class mrag_pvct(Variable):
    cerfa_field = {0: '5HW',
        1: '5IW',
        2: '5JW', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Plus-values agricoles (régime microBA) à court-terme'
    # start_date = date(2016, 1, 1)
    definition_period = YEAR


# (f5hx, f5ix, f5jx))
class mrag_pvce(Variable):
    cerfa_field = {0: '5HX',
        1: '5IX',
        2: '5JX', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Plus-values agricoles (régime microBA) à long-terme taxable selon droit commun'
    # start_date = date(2017, 1, 1)
    definition_period = YEAR


# (f5xo, f5yo, f5zo))
class mrag_mvct(Variable):
    cerfa_field = {0: '5XO',
        1: '5YO',
        2: '5ZO', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Moins-values agricoles (régime microBA) à court-terme'
    # start_date = date(2016, 1, 1)
    definition_period = YEAR


# (f5xn, f5yn, f5zn))
class mrag_mvlt(Variable):
    cerfa_field = {0: '5XN',
        1: '5YN',
        2: '5ZN', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Moins-values agricoles (régime microBA) à long-terme'
    # start_date = date(2016, 1, 1)
    definition_period = YEAR


# (f5he, f5ie, f5je))
class arag_pvce(Variable):
    cerfa_field = {0: '5HE',
        1: '5IE',
        2: '5JE', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = "Plus-values agricoles de cession taxables au régime de droit commun (Régime du bénéfice réel, revenus bénéficiant de l'abattement CGA ou viseur)"
    # start_date = date(2007, 1, 1)
    definition_period = YEAR


# TODO: vérif <=2012))  # (f5hk, f5lk, f5jk) codent autre chose sur d'autres années),
class nrag_pvce(Variable):
    cerfa_field = {0: '5HK',
        1: '5LK',
        2: '5JK', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = "Plus-values agricoles de cession taxables à 16% (Régime du bénéfice réel, revenus ne bénéficiant pas de l'abattement CGA ou viseur)"
    end = '2006-12-31'
    definition_period = YEAR


# (f5kq, f5lq, f5mq))
class mbic_pvce(Variable):
    cerfa_field = {0: '5KQ',
        1: '5LQ',
        2: '5MQ', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Plus-values industrielles et commerciales professionnelles imposables: plus-values de cession taxables selon droit commun (régime micro entreprise)'
    definition_period = YEAR


# (f5ke, f5le, f5me))
class abic_pvce(Variable):
    cerfa_field = {0: '5KE',
        1: '5LE',
        2: '5ME', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Plus-values industrielles et commerciales de cession taxables selon droit commun avec CGA ou viseur (régime du bénéfice réel)'
    definition_period = YEAR


# (f5kk, f5ik, f5mk)) TODO: autre 5KK 2005/20006
class nbic_pvce(Variable):
    cerfa_field = {0: '5IK',
        1: '5KK',
        2: '5MK', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus non commerciaux non professionnels exonérés sans AA (régime de la déclaration controlée)'
    # start_date = date(2008, 1, 1)
    definition_period = YEAR


# (f5nq, f5oq, f5pq))
class macc_pvce(Variable):
    cerfa_field = {0: '5NQ',
        1: '5OQ',
        2: '5PQ', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Plus-values industrielles et commerciales non professionnelles imposables: plus-values de cession taxables selon droit commun (régime micro entreprise)'
    definition_period = YEAR


# (f5ne, f5oe, f5pe))
class aacc_pvce(Variable):
    cerfa_field = {0: '5NE',
        1: '5OE',
        2: '5PE', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Plus-values industrielles et commerciales non professionnelles de cession taxables selon droit commun avec CGA ou viseur (régime du bénéfice réel)'
    definition_period = YEAR


# (f5nk, f5ok, f5pk)) TODO: 5NK 2005/2006
class nacc_pvce(Variable):
    cerfa_field = {0: '5NK',
        1: '5OK',
        2: '5PK', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Locations meublées non professionnelles: Revenus imposables sans CGA (régime du bénéfice réel)'
    # start_date = date(2009, 1, 1)
    definition_period = YEAR


# (f5kv, f5lv, f5mv))
class mncn_pvce(Variable):
    cerfa_field = {0: '5KV',
        1: '5LV',
        2: '5MV', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Plus-values non commerciales non professionnelles de cession taxables selon droit commun (régime déclaratif spécial ou micro BNC)'
    definition_period = YEAR


# (f5so, f5nt, f5ot))
class cncn_pvce(Variable):
    cerfa_field = {0: '5SO',
        1: '5NT',
        2: '5OT', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Plus-values nettes non commerciales non professionnelles taxables au taux de droit commun (régime de la déclaration controlée)'
    # start_date = date(2006, 1, 1)
    definition_period = YEAR


# (f5hr, f5ir, f5jr))
class mbnc_pvce(Variable):
    cerfa_field = {0: '5HR',
        1: '5IR',
        2: '5JR', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Plus-values non commerciales professionnelles de cession taxables selon droit commun(régime déclaratif spécial ou micro BNC)'
    definition_period = YEAR


# (f5qd, f5rd, f5sd))
class abnc_pvce(Variable):
    cerfa_field = {0: '5QD',
        1: '5RD',
        2: '5SD', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = "Plus-values non commerciaux professionnels de cession taxables selon droit commun (régime de la déclaration controlée, revenus bénéficiant de l'abattement association agrée ou viseur)"
    definition_period = YEAR


# (f5qj, f5rj, f5sj)) #TODO 5*J 2005/2006 (qui se transforme en 5*D...)
class nbnc_pvce(Variable):
    cerfa_field = {0: '5QJ',
        1: '5RJ',
        2: '5SJ', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Déficits industriels et commerciaux: locations meublées sans CGA (régime du bénéfice réel)'
    # start_date = date(2009, 1, 1)
    end = '2015-12-31'
    definition_period = YEAR


class frag_fore(Variable):
    cerfa_field = {0: '5HD',
        1: '5ID',
        2: '5JD', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus des exploitants forestiers (régime du forfait)'
    # start_date = date(2007, 1, 1)
    end = '2015-12-31'
    definition_period = YEAR


class arag_sjag(Variable):
    cerfa_field = {0: '5HZ',
        1: '5IZ',
        2: '5JZ', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Abattement pour les jeunes agriculteurs des revenus agricoles sans CGA (régime du bénéfice réel)'
    # start_date = date(2011, 1, 1)
    definition_period = YEAR


class abic_impm(Variable):
    cerfa_field = {0: '5HA',
        1: '5IA',
        2: '5JA', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Locations meublées imposables avec CGA ou viseur (régime du bénéfice réel pour les revenus industriels et commerciaux professionnels)'
    # start_date = date(2009, 1, 1)
    end = '2015-12-31'
    definition_period = YEAR


class nbic_impm(Variable):
    cerfa_field = {0: '5KA',
        1: '5LA',
        2: '5MA', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Locations meublées imposables sans CGA (régime du bénéfice réel)'
    # start_date = date(2009, 1, 1)
    end = '2015-12-31'
    definition_period = YEAR


class abic_defm(Variable):
    cerfa_field = {0: '5QA',
        1: '5RA',
        2: '5SA', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Déficits de locations meubléesavec CGA ou viseur (régime du bénéfice réel pour les revenus industriels et commerciaux professionnels)'
    # start_date = date(2009, 1, 1)
    end = '2015-12-31'
    definition_period = YEAR


class alnp_imps(Variable):
    cerfa_field = {0: '5NA',
        1: '5OA',
        2: '5PA', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Locations meublées non professionnelles imposables avec CGA ou viseur (régime du bénéfice réel)'
    # start_date = date(2009, 1, 1)
    definition_period = YEAR


class alnp_defs(Variable):
    cerfa_field = {0: '5NY',
        1: '5OY',
        2: '5PY', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Déficits de locations meublées non professionnelles avec CGA ou viseur (régime du bénéfice réel)'
    # start_date = date(2009, 1, 1)
    definition_period = YEAR


class nlnp_defs(Variable):
    cerfa_field = {0: '5NZ',
        1: '5OZ',
        2: '5PZ', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Déficits de locations meublées non professionnelles imposables sans CGA (régime du bénéfice réel)'
    # start_date = date(2009, 1, 1)
    end = '2010-12-31'
    # TODO: Toujours present dans brochure 2019 par ex.
    definition_period = YEAR


class abnc_proc(Variable):
    cerfa_field = {0: '5TF',
        1: '5UF',
        2: '5VF', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Honoraires de prospection commerciale exonérés avec CGA ou viseur (revenus non commerciaux professionnels, régime de la déclaration contrôlée)'
    # start_date = date(2009, 1, 1)
    end = '2017-12-31'
    definition_period = YEAR


class nbnc_proc(Variable):
    cerfa_field = {0: '5TI',
        1: '5UI',
        2: '5VI', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Honoraires de prospection commerciale exonérés sans CGA (revenus non commerciaux professionnels, régime de la déclaration contrôlée)'
    # start_date = date(2009, 1, 1)
    end = '2017-12-31'
    definition_period = YEAR


class mncn_exon(Variable):
    cerfa_field = {0: '5TH',
        1: '5UH',
        2: '5VH', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus nets exonérés non commerciaux non professionnels (régime déclaratif spécial ou micro BNC)'
    # start_date = date(2009, 1, 1)
    definition_period = YEAR


class cncn_exon(Variable):
    cerfa_field = {0: '5HK',
        1: '5JK',
        2: '5LK', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus nets exonérés non commerciaux non professionnels (régime de la déclaration contrôlée)'
    # start_date = date(2008, 1, 1)
    definition_period = YEAR


class cncn_aimp(Variable):
    cerfa_field = {0: '5JG',
        1: '5RF',
        2: '5SF', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus imposables non commerciaux non professionnels avec CGA (régime de la déclaration contrôlée)'
    # start_date = date(2007, 1, 1)
    definition_period = YEAR


class cncn_adef(Variable):
    cerfa_field = {0: '5JJ',
        1: '5RG',
        2: '5SG', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Déficits non commerciaux non professionnels avec CGA (régime de la déclaration contrôlée)'
    # start_date = date(2007, 1, 1)
    definition_period = YEAR


class cncn_info(Variable):
    cerfa_field = {0: '5TC',
        1: '5UC',
        2: '5VC', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Inventeurs et auteurs de logiciels : produits taxables aux taux de droit commun revenus non commerciaux non professionnels avec CGA (régime de la déclaration contrôlée)'
    # start_date = date(2009, 1, 1)
    end = '2018-12-31'
    definition_period = YEAR


class cncn_info_red1(Variable):
    cerfa_field = {0: '5TC',
        1: '5UC',
        2: '5VC', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Inventeurs et auteurs de logiciels : – produits taxables à 10%, déjà soumis aux CS, revenus non commerciaux non professionnels avec CGA (régime de la déclaration contrôlée)'
    # start_date = date(2019, 1, 1)
    definition_period = YEAR


class cncn_info_red2(Variable):
    cerfa_field = {0: '5QJ',
        1: '5RJ',
        2: '5SJ', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Inventeurs et auteurs de logiciels : – produits taxables à 10%, revenus non commerciaux non professionnels avec CGA (régime de la déclaration contrôlée)'
    # start_date = date(2019, 1, 1)
    definition_period = YEAR


class cncn_jcre(Variable):
    cerfa_field = {0: '5SV',
        1: '5SW',
        2: '5SX', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Jeunes créateurs : abattement de 50%, revenus non commerciaux non professionnels avec CGA (régime de la déclaration contrôlée)'
    # start_date = date(2006, 1, 1)
    definition_period = YEAR


class revimpres(Variable):
    cerfa_field = {0: '5HY',
        1: '5IY',
        2: '5JY', }
    value_type = int
    unit = 'currency'
    entity = Individu
    label = 'Revenus nets à imposer aux prélèvements sociaux'
    definition_period = YEAR


class pveximpres(Variable):
    cerfa_field = {0: '5HG',
        1: '5IG', }
    value_type = int
    entity = Individu
    label = 'Plus-values à long terme exonérées en cas de départ à la retraite à imposer aux prélèvements sociaux'
    # start_date = date(2006, 1, 1)
    definition_period = YEAR


class pvtaimpres(Variable):
    cerfa_field = {0: '5HZ',
        1: '5IZ',
        2: '5JZ', }
    value_type = int
    entity = Individu
    label = 'Plus-values à long terme taxables à 16% à la retraite à imposer aux prélèvements sociaux'
    end = '2009-12-31'
    definition_period = YEAR


class f5qf(Variable):
    cerfa_field = '5QF'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des revenus agricoles des années antérieures non encore déduits (n-6)'
    # start_date = date(2007, 1, 1)
    definition_period = YEAR


class f5qg(Variable):
    cerfa_field = '5QG'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des revenus agricoles des années antérieures non encore déduits (n-5)'
    # start_date = date(2007, 1, 1)
    definition_period = YEAR


class f5qn(Variable):
    cerfa_field = '5QN'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des revenus agricoles des années antérieures non encore déduits (n-4)'
    # start_date = date(2007, 1, 1)
    definition_period = YEAR


class f5qo(Variable):
    cerfa_field = '5QO'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des revenus agricoles des années antérieures non encore déduits (n-3)'
    # start_date = date(2007, 1, 1)
    definition_period = YEAR


class f5qp(Variable):
    cerfa_field = '5QP'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des revenus agricoles des années antérieures non encore déduits (n-2)'
    # start_date = date(2007, 1, 1)
    definition_period = YEAR


class f5qq(Variable):
    cerfa_field = '5QQ'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des revenus agricoles des années antérieures non encore déduits (n-1)'
    # start_date = date(2007, 1, 1)
    definition_period = YEAR


class f5ga(Variable):
    cerfa_field = '5GA'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des revenus de locations meublées non professionnelles années antérieures non encore déduits (n-10)'
    # start_date = date(2010, 1, 1)
    end = '2010-12-31'
    definition_period = YEAR


class f5gb(Variable):
    cerfa_field = '5GB'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des revenus de locations meublées non professionnelles années antérieures non encore déduits (n-9)'
    # start_date = date(2010, 1, 1)
    end = '2010-12-31'
    definition_period = YEAR


class f5gc(Variable):
    cerfa_field = '5GC'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des revenus de locations meublées non professionnelles années antérieures non encore déduits (n-8)'
    # start_date = date(2010, 1, 1)
    end = '2010-12-31'
    definition_period = YEAR


class f5gd(Variable):
    cerfa_field = '5GD'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des revenus de locations meublées non professionnelles années antérieures non encore déduits (n-7)'
    # start_date = date(2010, 1, 1)
    end = '2010-12-31'
    definition_period = YEAR


class f5ge(Variable):
    cerfa_field = '5GE'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des revenus de locations meublées non professionnelles années antérieures non encore déduits (n-6)'
    # start_date = date(2010, 1, 1)
    end = '2010-12-31'
    definition_period = YEAR


class f5gf(Variable):
    cerfa_field = '5GF'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des revenus de locations meublées non professionnelles années antérieures non encore déduits (n-5)'
    # start_date = date(2010, 1, 1)
    end = '2010-12-31'
    definition_period = YEAR


class f5gg(Variable):
    cerfa_field = '5GG'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des revenus de locations meublées non professionnelles années antérieures non encore déduits (n-4)'
    # start_date = date(2010, 1, 1)
    end = '2010-12-31'
    definition_period = YEAR


class f5gh(Variable):
    cerfa_field = '5GH'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des revenus de locations meublées non professionnelles années antérieures non encore déduits (n-3)'
    # start_date = date(2010, 1, 1)
    end = '2010-12-31'
    definition_period = YEAR


class f5gi(Variable):
    cerfa_field = '5GI'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des revenus de locations meublées non professionnelles années antérieures non encore déduits (n-2)'
    # start_date = date(2010, 1, 1)
    end = '2010-12-31'
    definition_period = YEAR


class f5gj(Variable):
    cerfa_field = '5GJ'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des revenus de locations meublées non professionnelles années antérieures non encore déduits (n-1)'
    # start_date = date(2010, 1, 1)
    end = '2010-12-31'
    definition_period = YEAR


class f5rn(Variable):
    cerfa_field = '5RN'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des revenus industriels et commerciaux non professionnelles années antérieures non encore déduits (n-6)'
    # start_date = date(2010, 1, 1)
    end = '2010-12-31'
    definition_period = YEAR


class f5ro(Variable):
    cerfa_field = '5RO'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des revenus industriels et commerciaux non professionnelles années antérieures non encore déduits (n-5)'
    definition_period = YEAR


class f5rp(Variable):
    cerfa_field = '5RP'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des revenus industriels et commerciaux non professionnelles années antérieures non encore déduits (n-4)'
    definition_period = YEAR


class f5rq(Variable):
    cerfa_field = '5RQ'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des revenus industriels et commerciaux non professionnelles années antérieures non encore déduits (n-3)'
    definition_period = YEAR


class f5rr(Variable):
    cerfa_field = '5RR'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des revenus industriels et commerciaux non professionnelles années antérieures non encore déduits (n-2)'
    definition_period = YEAR


class f5rw(Variable):
    cerfa_field = '5RW'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des revenus industriels et commerciaux non professionnelles années antérieures non encore déduits (n-1)'
    definition_period = YEAR


class f5ht(Variable):
    cerfa_field = '5HT'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des revenus non commerciaux non professionnelles années antérieures non encore déduits (n-6)'
    # start_date = date(2007, 1, 1)
    definition_period = YEAR


class f5it(Variable):
    cerfa_field = '5IT'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des revenus non commerciaux non professionnelles années antérieures non encore déduits (n-5)'
    # start_date = date(2007, 1, 1)
    definition_period = YEAR


class f5jt(Variable):
    cerfa_field = '5JT'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des revenus non commerciaux non professionnelles années antérieures non encore déduits (n-4)'
    # start_date = date(2007, 1, 1)
    definition_period = YEAR


class f5kt(Variable):
    cerfa_field = '5KT'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des revenus non commerciaux non professionnelles années antérieures non encore déduits (n-3)'
    # start_date = date(2007, 1, 1)
    definition_period = YEAR


class f5lt(Variable):
    cerfa_field = '5LT'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des revenus non commerciaux non professionnelles années antérieures non encore déduits (n-2)'
    # start_date = date(2007, 1, 1)
    definition_period = YEAR


class f5mt(Variable):
    cerfa_field = '5MT'
    value_type = int
    unit = 'currency'
    entity = FoyerFiscal
    label = 'Déficits des revenus non commerciaux non professionnelles années antérieures non encore déduits (n-1)'
    # start_date = date(2007, 1, 1)
    definition_period = YEAR


class f5sq(Variable):
    cerfa_field = '5SQ'
    value_type = int
    entity = Individu
    definition_period = YEAR
    label = 'Déficits des années antérieures non encore déduits'
    end = '2006-12-31'


# """
# Input variables
# """


# Input mensuel


class rpns_auto_entrepreneur_CA_achat_revente(Variable):
    value_type = float
    entity = Individu
    set_input = set_input_divide_by_period
    label = "Chiffre d'affaires en tant qu'auto-entrepreneur avec versement libératoire domaine ventes et assimilées "
    definition_period = MONTH

    def formula(individu, period):
        chiffre_affaire = individu('ebic_impv', period, options = [DIVIDE])

        return chiffre_affaire


class rpns_auto_entrepreneur_CA_bic(Variable):
    value_type = float
    entity = Individu
    set_input = set_input_divide_by_period
    label = "Chiffre d'affaires en tant qu'auto-entrepreneur avec versement libératoire domaine prestations de service et locations meublées "
    definition_period = MONTH

    def formula(individu, period):
        chiffre_affaire = individu('ebic_imps', period, options = [DIVIDE])

        return chiffre_affaire


class rpns_auto_entrepreneur_CA_bnc(Variable):
    value_type = float
    entity = Individu
    set_input = set_input_divide_by_period
    label = "Chiffre d'affaires en tant qu'auto-entrepreneur avec versement libératoire domaine non commercial"
    definition_period = MONTH

    def formula(individu, period):
        chiffre_affaire = individu('ebnc_impo', period, options = [DIVIDE])

        return chiffre_affaire


class rpns_auto_entrepreneur_chiffre_affaires(Variable):
    value_type = float
    entity = Individu
    set_input = set_input_divide_by_period
    label = "Chiffre d'affaires en tant qu'auto-entrepreneur avec versement libératoire"
    definition_period = MONTH

    def formula(individu, period):
        rpns_auto_entrepreneur_CA_achat_revente = individu('rpns_auto_entrepreneur_CA_achat_revente', period)
        rpns_auto_entrepreneur_CA_bic = individu('rpns_auto_entrepreneur_CA_bic', period)
        rpns_auto_entrepreneur_CA_bnc = individu('rpns_auto_entrepreneur_CA_bnc', period)

        return rpns_auto_entrepreneur_CA_achat_revente + rpns_auto_entrepreneur_CA_bic + rpns_auto_entrepreneur_CA_bnc

# Input annuel


class rpns_micro_entreprise_CA_bnc_imp(Variable):
    value_type = float
    entity = Individu
    label = "Chiffre d'affaires micro-entreprise domaine non commercial, revenus imposables"
    definition_period = YEAR

    def formula(individu, period):
        mbnc_impo = individu('mbnc_impo', period)
        mncn_impo = individu('mncn_impo', period)

        return mbnc_impo + mncn_impo


class rpns_micro_entreprise_CA_bnc_exon(Variable):
    value_type = float
    entity = Individu
    label = 'Revenus nets exonérés des  micro-entreprise domaine non commercial'
    definition_period = YEAR

    def formula(individu, period):
        mbnc_exon = individu('mbnc_exon', period)
        mncn_exon = individu('mncn_exon', period)

        return mbnc_exon + mncn_exon


class rpns_micro_entreprise_CA_bic_vente_imp(Variable):
    value_type = float
    entity = Individu
    label = "Chiffre d'affaires micro-entreprise domaine vente et assimilée, revenus imposables"
    definition_period = YEAR

    def formula(individu, period):
        mbic_impv = individu('mbic_impv', period)
        macc_impv = individu('macc_impv', period)

        return mbic_impv + macc_impv


class rpns_micro_entreprise_CA_bic_service_imp(Variable):
    value_type = float
    entity = Individu
    label = "Chiffre d'affaires micro-entreprise domaine service et locations, revenus imposables"
    definition_period = YEAR

    def formula(individu, period):
        mbic_imps = individu('mbic_imps', period)
        macc_imps = individu('macc_imps', period)
        aacc_imps = individu('aacc_imps', period)
        aacc_gits = individu('aacc_gits', period)
        nacc_meuc = individu('nacc_meuc', period)
        nacc_meup = individu('nacc_meup', period)

        return mbic_imps + macc_imps + aacc_imps + aacc_gits + nacc_meuc + nacc_meup


class rpns_micro_entreprise_bic_exon(Variable):
    value_type = float
    entity = Individu
    label = 'Revenus nets exonérés des micro-entreprise domaine bic'
    definition_period = YEAR

    def formula(individu, period):
        mbic_exon = individu('mbic_exon', period)
        macc_exon = individu('macc_exon', period)

        return mbic_exon + macc_exon


class rpns_micro_entreprise_chiffre_affaires(Variable):
    value_type = float
    entity = Individu
    label = "Chiffre d'affaires en micro-entreprise"
    definition_period = YEAR

    def formula(individu, period):
        rpns_micro_entreprise_CA_bnc_imp = individu('rpns_micro_entreprise_CA_bnc_imp', period)
        rpns_micro_entreprise_CA_bic_vente_imp = individu('rpns_micro_entreprise_CA_bic_vente_imp', period)
        rpns_micro_entreprise_CA_bic_service_imp = individu('rpns_micro_entreprise_CA_bic_service_imp', period)

        return rpns_micro_entreprise_CA_bnc_imp + rpns_micro_entreprise_CA_bic_vente_imp + rpns_micro_entreprise_CA_bic_service_imp


class TypesTnsTypeActivite(Enum):
    __order__ = 'achat_revente bic bnc'  # Needed to preserve the enum order in Python 2
    achat_revente = 'achat_revente'
    bic = 'bic'
    bnc = 'bnc'


# Input sur le dernier exercice. Par convention, sur l'année dernière.
class rpns_autres_revenus(Variable):
    value_type = float
    entity = Individu
    label = 'Autres revenus non salariés'
    definition_period = YEAR

    def formula(individu, period):
        abic_exon = individu('abic_exon', period)
        nbic_exon = individu('nbic_exon', period)
        abic_impn = individu('abic_impn', period)
        nbic_impn = individu('nbic_impn', period)
        aacc_exon = individu('aacc_exon', period)
        nacc_exon = individu('nacc_exon', period)
        aacc_impn = individu('aacc_impn', period)
        nacc_impn = individu('nacc_impn', period)
        alnp_imps = individu('alnp_imps', period)
        nacc_pvce = individu('nacc_pvce', period)
        abnc_exon = individu('abnc_exon', period)
        nbnc_exon = individu('nbnc_exon', period)
        abnc_impo = individu('abnc_impo', period)
        nbnc_impo = individu('nbnc_impo', period)
        cncn_exon = individu('cncn_exon', period)
        nbic_pvce = individu('nbic_pvce', period)
        cncn_aimp = individu('cncn_aimp', period)
        cncn_bene = individu('cncn_bene', period)

        return (abic_exon + nbic_exon + abic_impn + nbic_impn + aacc_exon
                + nacc_exon + aacc_impn + nacc_impn + alnp_imps + nacc_pvce
                + abnc_exon + nbnc_exon + abnc_impo + nbnc_impo + cncn_exon
                + nbic_pvce + cncn_aimp + cncn_bene)


class rpns_autres_revenus_chiffre_affaires(Variable):
    value_type = float
    entity = Individu
    set_input = set_input_divide_by_period
    label = "Chiffre d'affaire pour les TNS non agricoles autres que les AE et ME"
    definition_period = MONTH

    # les chiffres d'affaire ne sont pas dans les cases fiscales


class tns_autres_revenus_type_activite(Variable):
    value_type = Enum
    possible_values = TypesTnsTypeActivite
    default_value = TypesTnsTypeActivite.achat_revente
    entity = Individu
    label = "Type d'activité de l'entreprise non AE ni ME"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class tns_avec_employe(Variable):
    value_type = bool
    entity = Individu
    set_input = set_input_dispatch_by_period
    label = "Le TNS a au moins un employé. Ne s'applique pas pour les agricoles ni auto-entrepreneurs ni micro entreprise"
    definition_period = MONTH


# Input annuel


class rpns_benefice_exploitant_agricole(Variable):
    value_type = float
    entity = Individu
    label = 'Dernier bénéfice agricole'
    definition_period = YEAR

    def formula_2016_01_01(individu, period):
        rpns_revenus_microBA_agricole = individu('rpns_revenus_microBA_agricole', period)
        arag_exon = individu('arag_exon', period)
        arag_impg = individu('arag_impg', period)
        return rpns_revenus_microBA_agricole + arag_exon + arag_impg

    def formula(individu, period):
        rpns_revenus_forfait_agricole = individu('rpns_revenus_forfait_agricole', period)
        arag_exon = individu('arag_exon', period)
        arag_impg = individu('arag_impg', period)
        return rpns_revenus_forfait_agricole + arag_exon + arag_impg


# Computed variables


class travailleur_non_salarie(Variable):
    label = "L'individu a une activité professionnelle non salariée"
    value_type = bool
    entity = Individu
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period, parameters):
        this_year_and_last_year = Period(('year', period.start.offset('first-of', 'year'), 2)).offset(-1)
        rpns_auto_entrepreneur_chiffre_affaires = individu('rpns_auto_entrepreneur_chiffre_affaires', period) != 0
        rpns_micro_entreprise_chiffre_affaires = individu('rpns_micro_entreprise_chiffre_affaires', this_year_and_last_year, options = [ADD]) != 0
        rpns_autres_revenus = individu('rpns_autres_revenus', this_year_and_last_year, options = [ADD]) != 0
        rpns_benefice_exploitant_agricole = individu('rpns_benefice_exploitant_agricole', this_year_and_last_year, options = [ADD]) != 0
        rpns_autres_revenus_chiffre_affaires = individu('rpns_autres_revenus_chiffre_affaires', this_year_and_last_year, options = [ADD]) != 0

        result = (
            rpns_auto_entrepreneur_chiffre_affaires
            + rpns_micro_entreprise_chiffre_affaires
            + rpns_autres_revenus
            + rpns_benefice_exploitant_agricole
            + rpns_autres_revenus_chiffre_affaires
            )

        return result


class rpns_auto_entrepreneur_benefice(Variable):
    value_type = float
    label = "Bénéfice en tant qu'auto-entrepreneur avec versement libératoire"
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2008_01_01(individu, period, parameters):
        rpns_auto_entrepreneur_CA_achat_revente = individu('rpns_auto_entrepreneur_CA_achat_revente', period)
        rpns_auto_entrepreneur_CA_bic = individu('rpns_auto_entrepreneur_CA_bic', period)
        rpns_auto_entrepreneur_CA_bnc = individu('rpns_auto_entrepreneur_CA_bnc', period)

        bareme = parameters(period).impot_revenu.calcul_revenus_imposables.rpns.micro

        benefice = ((rpns_auto_entrepreneur_CA_achat_revente * (1 - bareme.microentreprise.regime_micro_bnc.marchandises.taux))
                    + (rpns_auto_entrepreneur_CA_bnc * (1 - bareme.microentreprise.regime_micro_bnc.taux))
                    + (rpns_auto_entrepreneur_CA_bic * (1 - bareme.microentreprise.regime_micro_bnc.services.taux)))

        return benefice


class rpns_micro_entreprise_benefice(Variable):
    value_type = float
    label = 'Bénéfice de la micro entreprise'
    entity = Individu
    definition_period = YEAR
    set_input = set_input_divide_by_period

    def formula_2008_01_01(individu, period, parameters):
        rpns_micro_entreprise_CA_bnc_imp = individu('rpns_micro_entreprise_CA_bnc_imp', period)
        rpns_micro_entreprise_CA_bic_vente_imp = individu('rpns_micro_entreprise_CA_bic_vente_imp', period)
        rpns_micro_entreprise_CA_bic_service_imp = individu('rpns_micro_entreprise_CA_bic_service_imp', period)
        rpns_micro_entreprise_CA_bnc_exon = individu('rpns_micro_entreprise_CA_bnc_exon', period)
        rpns_micro_entreprise_bic_exon = individu('rpns_micro_entreprise_bic_exon', period)

        bareme = parameters(period).impot_revenu.calcul_revenus_imposables.rpns.micro

        benefice = ((rpns_micro_entreprise_CA_bic_vente_imp * (1 - bareme.microentreprise.regime_micro_bnc.marchandises.taux))
                    + (rpns_micro_entreprise_CA_bnc_imp * (1 - bareme.microentreprise.regime_micro_bnc.taux))
                    + (rpns_micro_entreprise_CA_bic_service_imp * (1 - bareme.microentreprise.regime_micro_bnc.services.taux))
                    + rpns_micro_entreprise_CA_bnc_exon
                    + rpns_micro_entreprise_bic_exon)

        return benefice


# The following formulas take into account 'cotisation sociales'. However, it seems that for all prestations,
# the 'base ressources' are only using the 'benefice', without deducting the 'cotisation sociales'.
# Although this rule seems unfair towards independent workers, we are now applying it for all presations and therefore
# we are not using the following formulas for calculating prestations_sociales.
# This seemingly unfair method is however explained by the fact that the rate allowing to go from 'CA' to 'base_ressources' already takes into account
# the 'cotisations sociales', as both are directly computed from the same 'CA'. The main point of 'versement liberatoire' and 'micro-social' is to avoid to compute a true 'benefice',
# and so doing it in two steps would not be more accurate (it would be indeed unfair if the computed 'benefices' were actual 'benefices' before 'cotisations sociales' :
# in this case, one could however take a real 'benefice' taxation scheme).


class rpns_auto_entrepreneur_revenus_net(Variable):
    value_type = float
    label = "Revenu d'un auto-entrepreneur avec versement libératoire"
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2008_01_01(individu, period, parameters):
        rpns_auto_entrepreneur_benefice = individu('rpns_auto_entrepreneur_benefice', period)
        rpns_auto_entrepreneur_CA_achat_revente = individu('rpns_auto_entrepreneur_CA_achat_revente', period)
        rpns_auto_entrepreneur_CA_bic = individu('rpns_auto_entrepreneur_CA_bic', period)
        rpns_auto_entrepreneur_CA_bnc = individu('rpns_auto_entrepreneur_CA_bnc', period)
        bareme_cs_ae = parameters(period).taxation_societes.tns.auto_entrepreneur

        rpns_auto_entrepreneur_charges_sociales = (
            (bareme_cs_ae.achat_revente * rpns_auto_entrepreneur_CA_achat_revente)
            + (bareme_cs_ae.bic * rpns_auto_entrepreneur_CA_bic)
            + (bareme_cs_ae.bnc * rpns_auto_entrepreneur_CA_bnc))

        return rpns_auto_entrepreneur_benefice - rpns_auto_entrepreneur_charges_sociales


class rpns_micro_entreprise_revenus_net(Variable):
    value_type = float
    label = "Revenu d'un TNS dans une micro-entreprise"
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        rpns_micro_entreprise_benefice = individu('rpns_micro_entreprise_benefice', period, options = [DIVIDE])
        taux_cotisations_sociales = parameters(period).taxation_societes.tns.micro_entreprise.cotisations_sociales
        rpns_micro_entreprise_charges_sociales = rpns_micro_entreprise_benefice * taux_cotisations_sociales
        revenus = rpns_micro_entreprise_benefice - rpns_micro_entreprise_charges_sociales

        return revenus
